@@ -27,7 +27,11 @@ from .qtgui import (
     QVBoxLayout,
 )
 
-from mercurial import error
+from mercurial import (
+    error,
+    pycompat,
+    scmutil,
+)
 
 from ..util import hglib
 from ..util.i18n import _
@@ -73,11 +77,10 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
 
         possibleroots = []
         if minrev is not None:
-            parents = self.repo[minrev].parents()
+            parents = scmutil.revsymbol(self.repo, str(minrev)).parents()
             if parents:
                 for p in parents:
-                    text = hglib.tounicode(str(int(p)))
-                    possibleroots.append(text)
+                    possibleroots.append(u'%d' % p.rev())
             else:
                 possibleroots.append('null')
 
@@ -156,7 +159,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         # set default values
         self.prevtarget = None
         self.rev_combo.addItem(WD_PARENT)
-        self.rev_combo.addItems(map(hglib.tounicode,
+        self.rev_combo.addItems(pycompat.maplist(hglib.tounicode,
                                     hglib.namedbranches(self.repo)))
         tags = list(self.repo.tags())
         tags.sort(reverse=True)
@@ -217,7 +220,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
     def get_subrepos_present(self):
         rev = self.get_selected_rev()
         try:
-            ctx = self.repo[rev]
+            ctx = scmutil.revsymbol(self.repo, rev)
         except (error.LookupError, error.RepoLookupError):
             return False
         return '.hgsubstate' in ctx
@@ -246,7 +249,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             return path
         def remove_rev(path):
             l = ''
-            for i in xrange(self.rev_combo.count() - 1):
+            for i in pycompat.xrange(self.rev_combo.count() - 1):
                 l += unicode(self.rev_combo.itemText(i))
             revs = [rev[0] for rev in l]
             revs.append(wdrev)
@@ -272,7 +275,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
             text = wdrev
         else:
             try:
-                self.repo[hglib.fromunicode(text)]
+                scmutil.revsymbol(self.repo, hglib.fromunicode(text))
             except (error.RepoError, error.LookupError):
                 self.commandChanged.emit()
                 return
@@ -312,7 +315,7 @@ class ArchiveWidget(cmdui.AbstractCmdWidget):
         if not rev or not self.dest_edit.text():
             return False
         try:
-            return rev in self.repo
+            return scmutil.isrevsymbol(self.repo, rev)
         except error.LookupError:
             # ambiguous changeid
             return False
