@@ -5,21 +5,61 @@
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
 
-import cStringIO
-import os, re
+from __future__ import absolute_import
 
-from mercurial import util, patch, commands
-from mercurial import match as matchmod
+import os
+import re
 
-from tortoisehg.util import hglib
-from tortoisehg.util.patchctx import patchctx
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib, qscilib, lexers, visdiff, revert, rejects
-from tortoisehg.hgqt import filelistview, filedata, blockmatcher, manifestmodel
+from . import qsci as Qsci
+from .qtcore import (
+    QPoint,
+    QTimer,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAction,
+    QColor,
+    QDialog,
+    QFontMetrics,
+    QFrame,
+    QHBoxLayout,
+    QKeySequence,
+    QLabel,
+    QMenu,
+    QPainter,
+    QSplitter,
+    QStyle,
+    QToolBar,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4 import Qsci
+from mercurial import (
+    commands,
+    patch,
+    pycompat,
+    scmutil,
+    util,
+)
+
+from ..util import hglib
+from ..util.patchctx import patchctx
+from ..util.i18n import _
+from . import (
+    blockmatcher,
+    filedata,
+    filelistview,
+    lexers,
+    manifestmodel,
+    qscilib,
+    qtlib,
+    rejects,
+    revert,
+    visdiff,
+)
 
 # TODO
 # Add support for tools like TortoiseMerge that help resolve rejected chunks
@@ -45,7 +85,6 @@ class ChunksWidget(QWidget):
 
         layout = QVBoxLayout(self)
         layout.setSpacing(0)
-        layout.setMargin(0)
         layout.setContentsMargins(2, 2, 2, 2)
         self.setLayout(layout)
 
@@ -67,12 +106,11 @@ class ChunksWidget(QWidget):
         self.fileListFrame.setFrameShape(QFrame.NoFrame)
         vbox = QVBoxLayout()
         vbox.setSpacing(0)
-        vbox.setMargin(0)
+        vbox.setContentsMargins(0, 0, 0, 0)
         vbox.addWidget(self.filelist)
         self.fileListFrame.setLayout(vbox)
 
         self.diffbrowse = DiffBrowser(self.splitter)
-        self.diffbrowse.setFont(qtlib.getfont('fontdiff').font())
         self.diffbrowse.showMessage.connect(self.showMessage)
         self.diffbrowse.linkActivated.connect(self.linkActivated)
         self.diffbrowse.chunksSelected.connect(self.chunksSelected)
@@ -98,7 +136,7 @@ class ChunksWidget(QWidget):
             if icon:
                 act.setIcon(qtlib.geticon(icon))
             if key:
-                act.setShortcut(key)
+                qtlib.setContextMenuShortcut(act, key)
             if tip:
                 act.setStatusTip(tip)
             if cb:
@@ -190,7 +228,7 @@ class ChunksWidget(QWidget):
             if ret < 0:
                 ok = False
                 self.showMessage.emit(_('Patch failed to apply'))
-        except (patch.PatchError, EnvironmentError), err:
+        except (patch.PatchError, EnvironmentError) as err:
             ok = False
             self.showMessage.emit(hglib.tounicode(str(err)))
         rejfilere = re.compile(r'\b%s\.rej\b' % re.escape(wfile))
@@ -246,7 +284,7 @@ class ChunksWidget(QWidget):
         if isinstance(ctx, patchctx):
             repo.thgbackup(ctx._path)
             fp = util.atomictempfile(ctx._path, 'wb')
-            buf = cStringIO.StringIO()
+            buf = pycompat.bytesio()
             try:
                 if ctx._ph.comments:
                     buf.write('\n'.join(ctx._ph.comments))
@@ -285,10 +323,10 @@ class ChunksWidget(QWidget):
                 wlock = repo.wlock()
                 try:
                     # atomictemp can preserve file permission
-                    wf = repo.wopener(self.currentFile, 'wb', atomictemp=True)
+                    wf = repo.wvfs(self.currentFile, 'wb', atomictemp=True)
                     wf.write(self.diffbrowse.origcontents)
                     wf.close()
-                    fp = cStringIO.StringIO()
+                    fp = pycompat.bytesio()
                     chunks[0].write(fp)
                     for c in kchunks:
                         c.write(fp)
@@ -355,7 +393,7 @@ class ChunksWidget(QWidget):
         else:
             # Apply chunks to wfile
             repo.thgbackup(repo.wjoin(wfile))
-            fp = cStringIO.StringIO()
+            fp = pycompat.bytesio()
             for c in chunks:
                 c.write(fp)
             fp.seek(0)
@@ -412,9 +450,9 @@ class ChunksWidget(QWidget):
             else:
                 return []
         else:
-            buf = cStringIO.StringIO()
+            buf = pycompat.bytesio()
             diffopts = patch.diffopts(repo.ui, {'git':True})
-            m = matchmod.exact(repo.root, repo.root, [wfile])
+            m = scmutil.matchfiles(repo, [wfile])
             for p in patch.diff(repo, ctx.p1().node(), None, match=m,
                                 opts=diffopts):
                 buf.write(p)
@@ -428,7 +466,7 @@ class ChunksWidget(QWidget):
 
     @pyqtSlot(str, str)
     def displayFile(self, file, status):
-        if isinstance(file, (unicode, QString)):
+        if isinstance(file, unicode):
             file = hglib.fromunicode(file)
             status = hglib.fromunicode(status)
         if file:
@@ -465,10 +503,10 @@ class ChunksWidget(QWidget):
         ctx = self.ctx
         if isinstance(ctx, patchctx):
             # if patch mtime has not changed, it could return the same ctx
-            ctx = self.repo.changectx(ctx._path)
+            ctx = self.repo[ctx._path]
         else:
             self.repo.thginvalidate()
-            ctx = self.repo.changectx(ctx.node())
+            ctx = self.repo[ctx.node()]
         self.setContext(ctx)
 
     def loadSettings(self, qs, prefix):
@@ -536,16 +574,12 @@ class DiffBrowser(QFrame):
         self.searchbar.conditionChanged.connect(self.highlightText)
         self.addActions(self.searchbar.editorActions())
 
-        guifont = qtlib.getfont('fontlist').font()
         self.sumlabel = QLabel()
-        self.sumlabel.setFont(guifont)
         self.allbutton = QToolButton()
-        self.allbutton.setFont(guifont)
         self.allbutton.setText(_('All', 'files'))
         self.allbutton.setShortcut(QKeySequence.SelectAll)
         self.allbutton.clicked.connect(self.selectAll)
         self.nonebutton = QToolButton()
-        self.nonebutton.setFont(guifont)
         self.nonebutton.setText(_('None', 'files'))
         self.nonebutton.setShortcut(QKeySequence.New)
         self.nonebutton.clicked.connect(self.selectNone)
@@ -576,6 +610,7 @@ class DiffBrowser(QFrame):
         self.sci.setUtf8(True)
         self.sci.installEventFilter(qscilib.KeyPressInterceptor(self))
         self.sci.setCaretLineVisible(False)
+        self.sci.setFont(qtlib.getfont('fontdiff').font())
 
         self.sci.setMarginType(1, qsci.SymbolMargin)
         self.sci.setMarginLineNumbers(1, False)
@@ -635,7 +670,7 @@ class DiffBrowser(QFrame):
                 self.sci.markerAdd(chunk.mline, self.selected)
                 chunk.selected = True
                 self.countselected += 1
-                for i in xrange(*chunk.lrange):
+                for i in pycompat.xrange(*chunk.lrange):
                     self.sci.markerAdd(i, self.selcolor)
         self.updateSummary()
 
@@ -647,7 +682,7 @@ class DiffBrowser(QFrame):
                 self.sci.markerAdd(chunk.mline, self.unselected)
                 chunk.selected = False
                 self.countselected -= 1
-                for i in xrange(*chunk.lrange):
+                for i in pycompat.xrange(*chunk.lrange):
                     self.sci.markerDelete(i, self.selcolor)
         self.updateSummary()
 
@@ -665,13 +700,13 @@ class DiffBrowser(QFrame):
             self.sci.markerAdd(chunk.mline, self.unselected)
             chunk.selected = False
             self.countselected -= 1
-            for i in xrange(*chunk.lrange):
+            for i in pycompat.xrange(*chunk.lrange):
                 self.sci.markerDelete(i, self.selcolor)
         else:
             self.sci.markerAdd(chunk.mline, self.selected)
             chunk.selected = True
             self.countselected += 1
-            for i in xrange(*chunk.lrange):
+            for i in pycompat.xrange(*chunk.lrange):
                 self.sci.markerAdd(i, self.selcolor)
 
     def setContext(self, ctx):
@@ -742,15 +777,15 @@ class DiffBrowser(QFrame):
                     0, linkstart, 0, linkstart+len(forcedisplaymsg),
                     self._forceviewindicator)
             return
-        elif type(self._ctx.rev()) is str:
+        elif isinstance(self._ctx.rev(), str):
             chunks = self._ctx._files[filename]
         else:
-            header = patch.parsepatch(cStringIO.StringIO(fd.diff))[0]
+            header = patch.parsepatch(pycompat.bytesio(fd.diff))[0]
             chunks = [header] + header.hunks
 
         utext = []
         for chunk in chunks[1:]:
-            buf = cStringIO.StringIO()
+            buf = pycompat.bytesio()
             chunk.selected = False
             chunk.write(buf)
             chunk.lines = buf.getvalue().splitlines()
@@ -764,7 +799,7 @@ class DiffBrowser(QFrame):
             chunk.mline = start
             if start:
                 self.sci.markerAdd(start-1, self.divider)
-            for i in xrange(0,len(chunk.lines)):
+            for i in pycompat.xrange(0,len(chunk.lines)):
                 if start + i == chunk.mline:
                     self.sci.markerAdd(chunk.mline, self.unselected)
                 else:
