@@ -13,6 +13,10 @@ import struct
 import sys
 import time
 
+from mercurial import (
+    pycompat,
+)
+
 from .qtcore import (
     QBuffer,
     QIODevice,
@@ -24,6 +28,10 @@ from .qtcore import (
     pyqtSlot,
 )
 
+from mercurial import (
+    pycompat,
+)
+
 from ..util import (
     hglib,
     paths,
@@ -200,7 +208,7 @@ def _proccmdline(ui, exts):
     cmdline = list(paths.get_hg_command())
     for section, name, value in configs:
         cmdline.extend(('--config', '%s.%s=%s' % (section, name, value)))
-    return map(hglib.tounicode, cmdline)
+    return pycompat.maplist(hglib.tounicode, cmdline)
 
 
 class CmdProc(CmdWorker):
@@ -375,7 +383,7 @@ class CmdServer(CmdWorker):
         assert not self.isCommandRunning()
         try:
             data = hglib.fromunicode('\0'.join(cmdline))
-        except UnicodeEncodeError, inst:
+        except UnicodeEncodeError as inst:
             self._emitError(_('failed to encode command: %s') % inst)
             self._finishCommand(-1)
             return
@@ -453,7 +461,7 @@ class CmdServer(CmdWorker):
                     raise _ProtocolError(_('unexpected response on required '
                                            'channel %r') % ch)
                 chfunc(self, ch, dataorsize)
-        except _ProtocolError, inst:
+        except _ProtocolError as inst:
             self._emitError(inst.args[0])
             self.stopService()
         except Exception:
@@ -529,11 +537,11 @@ class CmdServer(CmdWorker):
             return
         try:
             data = hglib.fromunicode(text) + '\n'
-        except UnicodeEncodeError, inst:
+        except UnicodeEncodeError as inst:
             self._emitError(_('failed to encode input: %s') % inst)
             self.abortCommand()
             return
-        for start in xrange(0, len(data), size):
+        for start in pycompat.xrange(0, len(data), size):
             self._writeBlock(data[start:start + size])
 
     _idlechtable = {
@@ -836,7 +844,7 @@ class CmdAgent(QObject):
             self.busyChanged.emit(self.isBusy())
 
     def _cleanupWaitingSession(self):
-        for i in reversed(xrange(1, len(self._sessqueue))):
+        for i in reversed(pycompat.xrange(1, len(self._sessqueue))):
             sess = self._sessqueue[i]
             if sess.isFinished():
                 del self._sessqueue[i]
