@@ -5,28 +5,87 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import
+
+import cgi
 import os
 import re
 import tempfile
 import time
 
-from mercurial import util, error, phases
-from mercurial import obsolete  # delete if obsolete becomes enabled by default
+from .qsci import (
+    QsciAPIs,
+)
+from .qtcore import (
+    QSettings,
+    QSize,
+    QTimer,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAction,
+    QActionGroup,
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QFrame,
+    QHBoxLayout,
+    QKeySequence,
+    QLabel,
+    QLineEdit,
+    QMenu,
+    QMessageBox,
+    QPushButton,
+    QShortcut,
+    QSizePolicy,
+    QStyle,
+    QStyleFactory,
+    QStyleOptionToolButton,
+    QSplitter,
+    QToolBar,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+)
 
-from tortoisehg.util import hglib, i18n, shlib, wconfig
-from tortoisehg.util.i18n import _
+from mercurial import (
+    error,
+    obsolete,  # delete if obsolete becomes enabled by default
+    phases,
+    pycompat,
+    scmutil,
+)
+from mercurial.utils import (
+    dateutil,
+)
 
-from tortoisehg.hgqt.messageentry import MessageEntry
-from tortoisehg.hgqt import cmdcore, cmdui, thgrepo
-from tortoisehg.hgqt import qtlib, qscilib, status, branchop, revpanel
-from tortoisehg.hgqt import hgrcutil, lfprompt
-
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.Qsci import QsciAPIs
+from ..util import (
+    hglib,
+    i18n,
+    shlib,
+    wconfig,
+)
+from ..util.i18n import _
+from . import (
+    branchop,
+    cmdcore,
+    cmdui,
+    hgrcutil,
+    lfprompt,
+    qscilib,
+    qtlib,
+    revpanel,
+    status,
+    thgrepo,
+)
+from .messageentry import MessageEntry
 
 if os.name == 'nt':
-    from tortoisehg.util import bugtraq
+    from ..util import bugtraq
     _hasbugtraq = True
 else:
     _hasbugtraq = False
@@ -49,7 +108,7 @@ def readopts(ui):
 
 def commitopts2str(opts, mode='commit'):
     optslist = []
-    for opt, value in opts.iteritems():
+    for opt, value in opts.items():
         if opt in ['user', 'date', 'pushafter', 'autoinc',
                    'recurseinsubrepos']:
             if mode == 'merge' and opt == 'autoinc':
@@ -175,12 +234,10 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.setLayout(layout)
 
         vbox = QVBoxLayout()
-        vbox.setMargin(0)
         vbox.setSpacing(0)
         vbox.setContentsMargins(*(0,)*4)
 
         hbox = QHBoxLayout()
-        hbox.setMargin(0)
         hbox.setContentsMargins(*(0,)*4)
         tbar = QToolBar(_("Commit Dialog Toolbar"), self)
         tbar.setStyleSheet(qtlib.tbstylesheet)
@@ -219,7 +276,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 try:
                     parameters = self.opts['bugtraqparameters']
                     linktext = self.bugtraq.get_link_text(parameters)
-                except Exception, e:
+                except Exception as e:
                     tracker = self.opts['bugtraqplugin'].split(' ', 1)[1]
                     errormsg =  _('Failed to load issue tracker \'%s\': %s') \
                                  % (tracker, hglib.tounicode(str(e)))
@@ -331,7 +388,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         if not self.hasmqbutton:
             return 'commit'
         else:
-            pctx = self.repo.changectx('.')
+            pctx = self.repo['.']
             ispatch = 'qtip' in pctx.tags()
             if not ispatch:
                 # Set the button to Commit
@@ -344,15 +401,16 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 return 'qref'
 
     def commitSetupButton(self):
-        ispatch = lambda r: 'qtip' in r.changectx('.').tags()
-        notpatch = lambda r: 'qtip' not in r.changectx('.').tags()
+        ispatch = lambda r: 'qtip' in r['.'].tags()
+        notpatch = lambda r: 'qtip' not in r['.'].tags()
         def canamend(r):
             if ispatch(r):
                 return False
-            ctx = r.changectx('.')
+            ctx = r['.']
             return (ctx.phase() != phases.public) \
-                and len(r.changectx(None).parents()) < 2 \
-                and (obsolete._enabled or not ctx.children())
+                and len(r[None].parents()) < 2 \
+                and (obsolete.isenabled(self.repo, obsolete.allowunstableopt)
+                     or not ctx.children())
 
         acts = [
             ('commit', _('Commit changes'), _('Commit'), notpatch),
@@ -389,7 +447,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         committb.setBold()
         committb.setPopupMode(QToolButton.MenuButtonPopup)
         fmk = lambda s: committb.fontMetrics().width(hglib.tounicode(s[2]))
-        committb._width = max(map(fmk, acts)) + 4*committb.menuButtonWidth()
+        committb._width = (max(pycompat.maplist(fmk, acts))
+                           + 4*committb.menuButtonWidth())
 
         class CommitButtonMenu(QMenu):
             def __init__(self, parent, repo):
@@ -433,7 +492,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             selectedAction.setChecked(True)
         curraction = self.mqgroup.checkedAction()
         oldpctx = self.stwidget.pctx
-        pctx = self.repo.changectx('.')
+        pctx = self.repo['.']
         if curraction._name == 'qnew':
             self.pnlabel.setVisible(True)
             self.pnedit.setVisible(True)
@@ -514,7 +573,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             if branch in [p.branch() for p in repo[None].parents()]:
                 resp = 0
             else:
-                rev = repo[branch].rev()
+                # TODO: should look up only in branch namespace
+                rev = scmutil.revsymbol(repo, branch).rev()
                 resp = qtlib.CustomPrompt(_('Confirm Branch Change'),
                     _('Named branch "%s" already exists, '
                       'last used in revision %d\n'
@@ -596,7 +656,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 for tokentype, value in lexer.get_tokens(contents):
                     if tokentype in Token.Name and len(value) > 4:
                         tokens.add(value)
-            except ClassNotFound, TypeError:
+            except (ClassNotFound, TypeError):
                 pass
         except ImportError:
             pass
@@ -660,6 +720,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.lastCommitMsgs['amend'] = ('', False)  # avoid loading stale cache
         # refresh() may load/save the stale 'amend' message in commitSetAction()
         self.refresh()
+
+        # Updating after refreshWctx() leaves stale files in the commit list
+        # after amend + evolve + update.
+        if curraction._name == 'amend':
+            self.commitSetAction()
+
         self.stwidget.refreshWctx() # Trigger reload of working context
         # clear the last 'amend' message
         # do not clear the last 'commit' message because there are many cases
@@ -682,7 +748,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     @pyqtSlot()
     def refresh(self):
-        ispatch = self.repo.changectx('.').thgmqappliedpatch()
+        ispatch = self.repo['.'].thgmqappliedpatch()
         if not self.hasmqbutton:
             self.commitButtonEnable.emit(not ispatch)
         self.msgte.refresh(self.repo)
@@ -701,7 +767,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         opts = commitopts2str(self.opts)
         self.optionslabelfmt = _('<b>Selected Options:</b> %s')
         self.optionslabel.setText(self.optionslabelfmt
-                                  % Qt.escape(hglib.tounicode(opts)))
+                                  % cgi.escape(hglib.tounicode(opts)))
         self.optionslabel.setVisible(bool(opts))
 
         # Update parent csinfo widget
@@ -787,7 +853,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                         QMessageBox.Ok | QMessageBox.Cancel)
             if d != QMessageBox.Ok:
                 return
-        message = action.data().toString()
+        message = action.data()
         self.setMessage(message)
         self.msgte.setFocus()
 
@@ -807,13 +873,13 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         lpref = prefix + '/commit/' # local settings (splitter, etc)
         gpref = 'commit/'           # global settings (history, etc)
         # message history is stored in unicode
-        self.split.restoreState(s.value(lpref+'split').toByteArray())
+        self.split.restoreState(qtlib.readByteArray(s, lpref + 'split'))
         self.msgte.loadSettings(s, lpref+'msgte')
         self.stwidget.loadSettings(s, lpref+'status')
-        self.msghistory = list(s.value(gpref+'history-'+repoid).toStringList())
+        self.msghistory = qtlib.readStringList(s, gpref + 'history-' + repoid)
         self.msghistory = [unicode(m) for m in self.msghistory if m]
         self.updateRecentMessages()
-        self.userhist = map(unicode, s.value(gpref+'userhist').toStringList())
+        self.userhist = qtlib.readStringList(s, gpref + 'userhist')
         self.userhist = [u for u in self.userhist if u]
         try:
             curmsg = self.repo.vfs('cur-message.txt').read()
@@ -926,7 +992,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             merge = False
             files = self.stwidget.getChecked('MAR?!IS')
             # make list of files with partial change selections
-            for fname, c in self.stwidget.partials.iteritems():
+            for fname, c in self.stwidget.partials.items():
                 if c.excludecount > 0 and c.excludecount < len(c.hunks):
                     partials.append(fname)
             self.files = set(files + partials)
@@ -956,7 +1022,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             checkedUnknowns, lfiles = result
             if lfiles:
                 cmd = ['add', '--large', '--']
-                cmd.extend(map(hglib.escapepath, lfiles))
+                cmd.extend(pycompat.maplist(hglib.escapepath, lfiles))
                 commandlines.append(cmd)
         if checkedUnknowns:
             confirm = self.repo.ui.configbool('tortoisehg',
@@ -971,7 +1037,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 res = 0
             if res == 0:
                 cmd = ['add', '--']
-                cmd.extend(map(hglib.escapepath, checkedUnknowns))
+                cmd.extend(pycompat.maplist(hglib.escapepath, checkedUnknowns))
                 commandlines.append(cmd)
             else:
                 return
@@ -989,7 +1055,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 res = 0
             if res == 0:
                 cmd = ['remove', '--']
-                cmd.extend(map(hglib.escapepath, checkedMissing))
+                cmd.extend(pycompat.maplist(hglib.escapepath, checkedMissing))
                 commandlines.append(cmd)
             else:
                 return
@@ -1029,7 +1095,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             assert not self.stwidget.partials
 
         cmdline.append('--')
-        cmdline.extend(map(hglib.escapepath, self.files))
+        cmdline.extend(pycompat.maplist(hglib.escapepath, self.files))
         if len(repo[None].parents()) == 1:
             for fname in self.opts.get('autoinc', '').split(','):
                 fname = fname.strip()
@@ -1052,7 +1118,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.currentAction = action
         self.progress.emit(*cmdui.startProgress(_topicmap[action], ''))
         self.commitButtonEnable.emit(False)
-        ucmdlines = [map(hglib.tounicode, xs) for xs in cmdlines]
+        ucmdlines = [pycompat.maplist(hglib.tounicode, xs) for xs in cmdlines]
         self._cmdsession = sess = self._repoagent.runCommandSequence(ucmdlines,
                                                                      self)
         sess.commandFinished.connect(self.commandFinished)
@@ -1116,7 +1182,7 @@ class DetailsDialog(QDialog):
         try:
             val = hglib.tounicode(self.repo.ui.username())
             l.append(val)
-        except util.Abort:
+        except error.Abort:
             pass
         for name in userhistory:
             if name not in l:
@@ -1151,7 +1217,7 @@ class DetailsDialog(QDialog):
         self.datecb.toggled.connect(curdate.setEnabled)
         self.datecb.toggled.connect(lambda s: s and curdate.setFocus())
         curdate.clicked.connect( lambda: self.datele.setText(
-                hglib.tounicode(hglib.displaytime(util.makedate()))))
+                hglib.tounicode(hglib.displaytime(dateutil.makedate()))))
         if opts.get('date'):
             self.datele.setText(opts['date'])
             self.datecb.setChecked(True)
@@ -1264,7 +1330,7 @@ class DetailsDialog(QDialog):
                 except KeyError:
                     pass
             wconfig.writefile(cfg, fn)
-        except IOError, e:
+        except IOError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(e), parent=self)
 
@@ -1287,7 +1353,7 @@ class DetailsDialog(QDialog):
                 except KeyError:
                     pass
             wconfig.writefile(cfg, fn)
-        except IOError, e:
+        except IOError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(e), parent=self)
 
@@ -1310,7 +1376,7 @@ class DetailsDialog(QDialog):
                 except KeyError:
                     pass
             wconfig.writefile(cfg, fn)
-        except IOError, e:
+        except IOError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(e), parent=self)
 
@@ -1333,7 +1399,7 @@ class DetailsDialog(QDialog):
                 except KeyError:
                     pass
             wconfig.writefile(cfg, fn)
-        except IOError, e:
+        except IOError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(e), parent=self)
 
@@ -1342,8 +1408,8 @@ class DetailsDialog(QDialog):
         if self.datecb.isChecked():
             date = hglib.fromunicode(self.datele.text())
             try:
-                util.parsedate(date)
-            except error.Abort, e:
+                dateutil.parsedate(date)
+            except error.ParseError as e:
                 if e.hint:
                     err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
                                                 hglib.tounicode(e.hint))
@@ -1363,7 +1429,7 @@ class DetailsDialog(QDialog):
         if not user:
             try:
                 self.repo.ui.username()
-            except util.Abort, e:
+            except error.Abort as e:
                 if e.hint:
                     err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
                                                 hglib.tounicode(e.hint))
@@ -1405,7 +1471,7 @@ class CommitDialog(QDialog):
         self.opts = opts
 
         layout = QVBoxLayout()
-        layout.setMargin(0)
+        layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
 
         toplayout = QVBoxLayout()
@@ -1439,7 +1505,7 @@ class CommitDialog(QDialog):
                                               parent=self)
 
         s = QSettings()
-        self.restoreGeometry(s.value('commit/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'commit/geom'))
         commit.loadSettings(s, 'committool')
         repoagent.repositoryChanged.connect(self.updateUndo)
         commit.commitComplete.connect(self.postcommit)
