@@ -16,14 +16,35 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import
+
 import re
 
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib
-from tortoisehg.util import hglib
+from .qtcore import (
+    QSettings,
+    Qt,
+)
+from .qtgui import (
+    QComboBox,
+    QDialog,
+    QFormLayout,
+    QFrame,
+    QHBoxLayout,
+    QLabel,
+    QLineEdit,
+    QListWidget,
+    QListWidgetItem,
+    QPushButton,
+    QTableWidget,
+    QTableWidgetItem,
+    QVBoxLayout,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util import hglib
+from ..util.i18n import _
+from . import qtlib
+
+DEFAULTICONNAME = 'tools-spanner-hammer'
 
 
 class ToolsFrame(QFrame):
@@ -49,7 +70,7 @@ class ToolsFrame(QFrame):
             toolTip=_('Select the toolbar or menu to change'))
 
         def selectlocation(index):
-            location = self.locationcombo.itemData(index).toString()
+            location = self.locationcombo.itemData(index)
             for widget in self.widgets:
                 if widget.location == location:
                     widget.removeInvalid(self.value())
@@ -116,7 +137,7 @@ class ToolsFrame(QFrame):
 
     def getCurrentToolList(self):
         index = self.locationcombo.currentIndex()
-        location = self.locationcombo.itemData(index).toString()
+        location = self.locationcombo.itemData(index)
         for widget in self.widgets:
             if widget.location == location:
                 return widget
@@ -165,7 +186,7 @@ class ToolsFrame(QFrame):
                 toolname, toolconfig = td.value()
                 icon = toolconfig.get('icon', '')
                 if not icon:
-                    icon = td._defaulticonname
+                    icon = DEFAULTICONNAME
                 item = QListWidgetItem(qtlib.geticon(icon), toolname)
                 gtl.takeItem(row)
                 gtl.insertItem(row, item)
@@ -251,7 +272,7 @@ class ToolsFrame(QFrame):
 
         # 2. Save the new guidefs
         for n, toollistwidget in enumerate(self.widgets):
-            toollocation = self.locationcombo.itemData(n).toString()
+            toollocation = self.locationcombo.itemData(n)
             if not toollistwidget.isDirty():
                 continue
             emitChanged = True
@@ -430,8 +451,11 @@ class HooksFrame(QFrame):
         hooks = self.curvalue
         for hooktype in hooks:
             for keyname in hooks[hooktype]:
-                try:
+                if keyname:
                     keyname = '%s.%s' % (hooktype, keyname)
+                else:
+                    keyname = hooktype
+                try:
                     del ini[section][keyname]
                 except KeyError:
                     pass
@@ -521,8 +545,7 @@ class ToolListBox(QListWidget):
         # Enable drag and drop to reorder the tools
         self.setDragEnabled(True)
         self.setDragDropMode(self.InternalMove)
-        if PYQT_VERSION >= 0x40700:
-            self.setDefaultDropAction(Qt.MoveAction)
+        self.setDefaultDropAction(Qt.MoveAction)
 
     def _guidef2toollist(self, guidef):
         toollist = []
@@ -553,7 +576,7 @@ class ToolListBox(QListWidget):
             item = text
         else:
             if not icon:
-                icon = CustomToolConfigDialog._defaulticonname
+                icon = DEFAULTICONNAME
             if isinstance(icon, str):
                 icon = qtlib.geticon(icon)
             item = QListWidgetItem(icon, text)
@@ -585,7 +608,7 @@ class ToolListBox(QListWidget):
         self.curvalue = curvalue
 
     def value(self):
-        return self._toollist2guidef(self.values())
+        return self._toollist2guidef(list(self.values()))
 
     def isDirty(self):
         return self.value() != self.curvalue
@@ -642,7 +665,7 @@ class CustomConfigDialog(QDialog):
     def value(self):
         return None
 
-    def _genCombo(self, items, selecteditem=None):
+    def _genCombo(self, items, selecteditem=None, tooltips=None):
         index = 0
         if selecteditem:
             try:
@@ -653,6 +676,9 @@ class CustomConfigDialog(QDialog):
         combo.addItems(items)
         if index:
             combo.setCurrentIndex(index)
+        if tooltips:
+            for idx, tooltip in enumerate(tooltips):
+                combo.setItemData(idx, tooltip, Qt.ToolTipRole)
         return combo
 
     def _addConfigItem(self, parent, label, configwidget, tooltip=None):
@@ -674,7 +700,8 @@ class CustomConfigDialog(QDialog):
     def _readsettings(self):
         s = QSettings()
         if self.dialogname:
-            self.restoreGeometry(s.value(self.dialogname + '/geom').toByteArray())
+            self.restoreGeometry(
+                qtlib.readByteArray(s, self.dialogname + '/geom'))
         return s
 
     def _writesettings(self):
@@ -698,14 +725,13 @@ class CustomToolConfigDialog(CustomConfigDialog):
                        (_('Applied patches'), 'applied'),
                        (_('Applied patches or qparent'), 'qgoto'),
                        ]
-    _defaulticonname = 'tools-spanner-hammer'
     _defaulticonstring = _('<default icon>')
 
     def __init__(self, parent=None, toolname=None, toolconfig={}):
         super(CustomToolConfigDialog, self).__init__(parent,
             dialogname='customtools',
             windowTitle=_('Configure Custom Tool'),
-            windowIcon=qtlib.geticon(self._defaulticonname))
+            windowIcon=qtlib.geticon(DEFAULTICONNAME))
 
         vbox = self.formvbox
 
@@ -724,15 +750,22 @@ class CustomToolConfigDialog(CustomConfigDialog):
             QLineEdit(command), _('The command that will be executed.\n'
             'To execute a Mercurial command use "hg" (rather than "hg.exe") '
             'as the executable command.\n'
-            'You can use several {VARIABLES} to compose your command:\n'
+            'You can use several {VARIABLES} to compose your command.\n'
+            'Common variables:\n'
             '- {ROOT}: The path to the current repository root.\n'
-            '- {REV} / {REVID}: the selected revision number / '
-            'hexadecimal revision id hash respectively.\n'
+            '- {REV} / {REVID}: Selected revisions numbers / hexadecimal'
+            ' revision id hashes respectively formatted as a revset'
+            ' expression.\n'
             '- {SELECTEDFILES}: The list of files selected by the user on the '
             'revision details file list.\n'
-            '- {FILES}: The list of files touched by the selected revision.\n'
+            '- {FILES}: The list of files touched by the selected revisions.\n'
             '- {ALLFILES}: All the files tracked by Mercurial on the selected'
-            ' revision.'))
+            ' revisions.\n'
+            'Pair selection variables:\n'
+            '- {REV_A} / {REVID_A}: the first selected revision number / '
+            'hexadecimal revision id hash respectively.\n'
+            '- {REV_B} / {REVID_B}: the second selected revision number / '
+            'hexadecimal revision id hash respectively.\n'))
         self.workingdir = self._addConfigItem(vbox, _('Working Directory'),
             QLineEdit(workingdir),
             _('The directory where the command will be executed.\n'
@@ -757,7 +790,7 @@ class CustomToolConfigDialog(CustomConfigDialog):
             ico = self._defaulticonstring
         elif ico not in iconnames:
             combo.addItem(qtlib.geticon(ico), ico)
-        combo.addItem(qtlib.geticon(self._defaulticonname),
+        combo.addItem(qtlib.geticon(DEFAULTICONNAME),
                       self._defaulticonstring)
         for name in iconnames:
             combo.addItem(qtlib.geticon(name), name)
@@ -839,6 +872,90 @@ class HookConfigDialog(CustomConfigDialog):
         'tag',
         'update',
     )
+
+    _hooktooltips = (
+        _('Run after a changegroup has been added via push, pull or unbundle. '
+            'ID of the first new changeset is in <tt>$HG_NODE</tt> and last in '
+            '<tt>$HG_NODE_LAST</tt>. URL from which changes came is in '
+            '<tt>$HG_URL</tt>.'),
+        _('Run after a changeset has been created in the local repository. ID '
+            'of the newly created changeset is in <tt>$HG_NODE</tt>. Parent '
+            'changeset IDs are in <tt>$HG_PARENT1</tt> and '
+            '<tt>$HG_PARENT2</tt>.'),
+        _('Run after a changeset has been pulled, pushed, or unbundled into '
+            'the local repository. The ID of the newly arrived changeset is in '
+            '<tt>$HG_NODE</tt>. URL that was source of changes came is in '
+            '<tt>$HG_URL</tt>.'),
+        _('Run after sending changes from local repository to another. ID of '
+            'first changeset sent is in <tt>$HG_NODE</tt>. Source of operation '
+            'is in <tt>$HG_SOURCE</tt>.'),
+        _('Run before a changegroup is added via push, pull or unbundle. Exit '
+            'status 0 allows the changegroup to proceed. Non-zero status will '
+            'cause the push, pull or unbundle to fail. URL from which changes '
+            'will come is in <tt>$HG_URL</tt>.'),
+        _('Run before starting a local commit. Exit status 0 allows the commit '
+            'to proceed. Non-zero status will cause the commit to fail. Parent '
+            'changeset IDs are in <tt>$HG_PARENT1</tt> and '
+            '<tt>$HG_PARENT2</tt>.'),
+        _('Run before listing pushkeys (like bookmarks) in the repository. '
+            'Non-zero status will cause failure. The key namespace is in '
+            '<tt>$HG_NAMESPACE</tt>.'),
+        _('Run before collecting changes to send from the local repository to '
+            'another. Non-zero status will cause failure. This lets you '
+            'prevent pull over HTTP or SSH. Also prevents against local pull, '
+            'push (outbound) or bundle commands, but not effective, since you '
+            'can just copy files instead then. Source of operation is in '
+            '<tt>$HG_SOURCE</tt>. If "serve", operation is happening on behalf '
+            'of remote SSH or HTTP repository. If "push", "pull" or "bundle", '
+            'operation is happening on behalf of repository on same system.'),
+        _('Run before a pushkey (like a bookmark) is added to the repository. '
+            'Non-zero status will cause the key to be rejected. The key '
+            'namespace is in <tt>$HG_NAMESPACE</tt>, the key is in '
+            '<tt>$HG_KEY</tt>, the old value (if any) is in <tt>$HG_OLD</tt>, '
+            'and the new value is in <tt>$HG_NEW</tt>.'),
+        _('Run before creating a tag. Exit status 0 allows the tag to be '
+            'created. Non-zero status will cause the tag to fail. ID of '
+            'changeset to tag is in <tt>$HG_NODE</tt>. Name of tag is in '
+            '<tt>$HG_TAG</tt>. Tag is local if <tt>$HG_LOCAL=1</tt>, in '
+            'repository if <tt>$HG_LOCAL=0</tt>.'),
+        _('Run after a changegroup has been added via push, pull or unbundle, '
+            'but before the transaction has been committed. Changegroup is '
+            'visible to hook program. This lets you validate incoming changes '
+            'before accepting them. Passed the ID of the first new changeset '
+            'in <tt>$HG_NODE</tt> and last in <tt>$HG_NODE_LAST</tt>. Exit '
+            'status 0 allows the transaction to commit. Non-zero status will '
+            'cause the transaction to be rolled back and the push, pull or '
+            'unbundle will fail. URL that was source of changes is in '
+            '<tt>$HG_URL</tt>.'),
+        _('Run after a changeset has been created but the transaction not yet '
+            'committed. Changeset is visible to hook program. This lets you '
+            'validate commit message and changes. Exit status 0 allows the '
+            'commit to proceed. Non-zero status will cause the transaction to '
+            'be rolled back. ID of changeset is in <tt>$HG_NODE</tt>. Parent '
+            'changeset IDs are in <tt>$HG_PARENT1</tt> and '
+            '<tt>$HG_PARENT2</tt>.'),
+        _('Run before updating the working directory. Exit status 0 allows the '
+            'update to proceed. Non-zero status will prevent the update. '
+            'Changeset ID of first new parent is in <tt>$HG_PARENT1</tt>. '
+            'If merge, ID of second new parent is in <tt>$HG_PARENT2</tt>.'),
+        _('Run after listing pushkeys (like bookmarks) in the repository. The '
+            'key namespace is in <tt>$HG_NAMESPACE</tt>. <tt>$HG_VALUES</tt> '
+            'is a dictionary containing the keys and values.'),
+        _('Run after a pushkey (like a bookmark) is added to the repository. '
+            'The key namespace is in <tt>$HG_NAMESPACE</tt>, the key is in '
+            '<tt>$HG_KEY</tt>, the old value (if any) is in <tt>$HG_OLD</tt>, '
+            'and the new value is in <tt>$HG_NEW</tt>.'),
+        _('Run after a tag is created. ID of tagged changeset is in '
+            '<tt>$HG_NODE</tt>. Name of tag is in <tt>$HG_TAG</tt>. Tag is '
+            'local if <tt>$HG_LOCAL=1</tt>, in repository if '
+            '<tt>$HG_LOCAL=0</tt>.'),
+        _('Run after updating the working directory. Changeset ID of first new '
+            'parent is in <tt>$HG_PARENT1</tt>. If merge, ID of second new '
+            'parent is in <tt>$HG_PARENT2</tt>. If the update succeeded, '
+            '<tt>$HG_ERROR=0</tt>. If the update failed (e.g. because '
+            'conflicts not resolved), <tt>$HG_ERROR=1</tt>.'),
+    )
+
     _rehookname = re.compile('^[^=\s]*$')
 
     def __init__(self, parent=None, hooktype=None, command='', hookname=''):
@@ -848,7 +965,7 @@ class HookConfigDialog(CustomConfigDialog):
             windowIcon=qtlib.geticon('tools-hooks'))
 
         vbox = self.formvbox
-        combo = self._genCombo(self._hooktypes, hooktype)
+        combo = self._genCombo(self._hooktypes, hooktype, self._hooktooltips)
         self.hooktype = self._addConfigItem(vbox, _('Hook type'),
             combo, _('Select when your command will be run'))
         self.name = self._addConfigItem(vbox, _('Tool name'),
@@ -874,40 +991,3 @@ class HookConfigDialog(CustomConfigDialog):
         if not command:
             return _('You must set a command to run.')
         return '' # No error
-
-
-def addCustomToolsSubmenu(menu,
-        ui, location, make, slot, label=_('Custom Tools')):
-    '''
-    Add a custom tools submenu to an existing menus
-
-    This can be used, for example, to add the custom tools submenu to the
-    different file context menus
-    '''
-    tools, toollist = hglib.tortoisehgtools(ui,
-        selectedlocation=location)
-    if not tools:
-        return
-    submenu = menu.addMenu(label)
-    submenu.triggered.connect(slot)
-    emptysubmenu = True
-    for name in toollist:
-        if name == '|':
-            submenu.addSeparator()
-            continue
-        info = tools.get(name, None)
-        if info is None:
-            continue
-        command = info.get('command', None)
-        if not command:
-            continue
-        label = info.get('label', name)
-        icon = info.get('icon', CustomToolConfigDialog._defaulticonname)
-        status = info.get('status', 'MAR!C?S')
-        a = make(label, None, frozenset(status),
-            icon=icon, inmenu=submenu)
-        if a is not None:
-            a.setData(name)
-            emptysubmenu = False
-    if emptysubmenu:
-        menu.removeAction(submenu.menuAction())
