@@ -5,25 +5,49 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-import glob, os, shlex
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.Qsci import QsciScintilla
+import os
 
-from mercurial import commands, util
+from .qsci import (
+    QsciScintilla,
+)
+from .qtcore import (
+    QIODevice,
+    QProcess,
+    QTimer,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QColor,
+    QDockWidget,
+    QStackedWidget,
+    QVBoxLayout,
+    QWidget,
+)
 
-from tortoisehg.hgqt import cmdui, qtlib
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
+from mercurial import (
+    commands,
+    pycompat,
+    util,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdui,
+    qtlib,
+)
 
 class _LogWidgetForConsole(cmdui.LogWidget):
     """Wrapped LogWidget for ConsoleWidget"""
 
-    returnPressed = pyqtSignal(unicode)
+    returnPressed = pyqtSignal(str)
     """Return key pressed when cursor is on prompt line"""
-    historyRequested = pyqtSignal(unicode, int)  # keyword, direction
-    completeRequested = pyqtSignal(unicode)
+    historyRequested = pyqtSignal(str, int)  # keyword, direction
+    completeRequested = pyqtSignal(str)
 
     _prompt = '% '
 
@@ -288,7 +312,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
         for name in ('openPrompt', 'closePrompt', 'clear'):
             setattr(self, name, getattr(self._logwidget, name))
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def historySearch(self, text, direction):
         cmdline, idx = _searchhistory(self._commandHistory, unicode(text),
                                       direction, self._commandIdx)
@@ -313,6 +337,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
             matchinfo = {}
             for cmdspec in cmdtable:
                 for cmdname in cmdspec.split('|'):
+                    # TODO: switch to hg>=4.8 (fa88170c10bb) syntax
                     if cmdname[0] == '^':
                         cmdname = cmdname[1:]
                     if cmdname.startswith(cmdstart):
@@ -325,7 +350,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
             basecmdline = '%s %%s' % (cmdtype)
             matches = [basecmdline % c for c in matchingcmds]
         else:
-            scmdtype = matchingcmds.keys()[0]
+            scmdtype = list(matchingcmds)[0]
             cmdspec = matchingcmds[scmdtype]
             opts = cmdtable[cmdspec][1]
             def findcmdopt(cmdopt):
@@ -350,7 +375,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
                     matches = [basecmdline % opt[1] for opt in cmdopts]
         return sorted(matches)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def completeCommandText(self, text):
         """Show the list of history or known commands matching the search text
 
@@ -417,7 +442,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
         text = hglib.tounicode(self._extproc.readAllStandardError().data())
         self._logwidget.appendLog(text, 'ui.warning')
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def appendLog(self, msg, label):
         """Append log text to the last line while keeping the prompt line"""
         self._logwidget.clearPrompt()
@@ -446,13 +471,13 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
         else:
             self.openPrompt()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _runcommand(self, cmdline):
         cmdline = unicode(cmdline)
         self._commandIdx = 0
         try:
-            args = list(self._parsecmdline(cmdline))
-        except ValueError, e:
+            args = hglib.parsecmdline(cmdline, self._workingDirectory())
+        except ValueError as e:
             self.closePrompt()
             self._logwidget.appendLog(unicode(e) + '\n', 'ui.error')
             self.openPrompt()
@@ -470,24 +495,6 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
         except KeyError:
             return self._runextcommand(cmdline)
 
-    def _parsecmdline(self, cmdline):
-        """Split command line string to imitate a unix shell"""
-        try:
-            # shlex can't process unicode on Python < 2.7.3
-            args = shlex.split(cmdline.encode('utf-8'))
-        except ValueError, e:
-            raise ValueError(_('command parse error: %s') % e)
-        for e in args:
-            e = util.expandpath(e).decode('utf-8')
-            if util.any(c in e for c in '*?[]'):
-                expanded = glob.glob(os.path.join(self._workingDirectory(), e))
-                if not expanded:
-                    raise ValueError(_('no matches found: %s') % e)
-                for p in expanded:
-                    yield p
-            else:
-                yield e
-
     def _runextcommand(self, cmdline):
         self._extproc.setWorkingDirectory(self._workingDirectory())
         self._extproc.start(cmdline, QIODevice.ReadOnly)
@@ -503,7 +510,7 @@ class ConsoleWidget(QWidget, qtlib.TaskWidget):
             if self.repoRootPath():
                 args = ['-R', self.repoRootPath()] + args
             # TODO: show errors
-            run.dispatch(map(hglib.fromunicode, args))
+            run.dispatch(pycompat.maplist(hglib.fromunicode, args))
         finally:
             self.openPrompt()
 
@@ -552,7 +559,7 @@ class LogDockWidget(QDockWidget):
         self.setFocusProxy(w)
 
     def _findConsoleFor(self, root):
-        for i in xrange(self._consoles.count()):
+        for i in pycompat.xrange(self._consoles.count()):
             w = self._consoles.widget(i)
             if w.repoRootPath() == root:
                 return w
@@ -564,14 +571,14 @@ class LogDockWidget(QDockWidget):
         self._consoles.addWidget(w)
         return w
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _createConsoleFor(self, root):
         root = unicode(root)
         repoagent = self._repomanager.repoAgent(root)
         assert repoagent
         self._createConsole(repoagent)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _destroyConsoleFor(self, root):
         root = unicode(root)
         w = self._findConsoleFor(root)
