@@ -5,15 +5,39 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from __future__ import absolute_import
 
 import os
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib, cmdcore, cmdui, resolve, thgrepo, wctxcleaner
-from tortoisehg.hgqt import csinfo, cslist
+from .qtcore import (
+    QSettings,
+    QTimer,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QGroupBox,
+    QMessageBox,
+    QVBoxLayout,
+)
+
+from mercurial import scmutil
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    csinfo,
+    cslist,
+    qtlib,
+    resolve,
+    thgrepo,
+    wctxcleaner,
+)
 
 BB = QDialogButtonBox
 
@@ -27,11 +51,11 @@ class GraftDialog(QDialog):
 
         self._repoagent = repoagent
         self._cmdsession = cmdcore.nullCmdSession()
-        self._graftstatefile = self.repo.join('graftstate')
+        self._graftstatefile = self.repo.vfs.join('graftstate')
         self.valid = True
 
         def cleanrevlist(revlist):
-            return [self.repo[rev].rev() for rev in revlist]
+            return [scmutil.revsymbol(self.repo, rev).rev() for rev in revlist]
         self.sourcelist = cleanrevlist(opts.get('source', ['.']))
         currgraftrevs = self.graftstate()
         if currgraftrevs:
@@ -133,18 +157,18 @@ class GraftDialog(QDialog):
         ui = self.repo.ui
         qs = QSettings()
         qs.beginGroup('graft')
-        for n, w in self._optchks.iteritems():
+        for n, w in self._optchks.items():
             if n == 'autoresolve':
                 w.setChecked(ui.configbool('tortoisehg', n,
-                                           qs.value(n, True).toBool()))
+                                           qtlib.readBool(qs, n, True)))
             else:
-                w.setChecked(qs.value(n).toBool())
+                w.setChecked(qtlib.readBool(qs, n))
         qs.endGroup()
 
     def _writeSettings(self):
         qs = QSettings()
         qs.beginGroup('graft')
-        for n, w in self._optchks.iteritems():
+        for n, w in self._optchks.items():
             qs.setValue(n, w.isChecked())
         qs.endGroup()
 
@@ -164,9 +188,10 @@ class GraftDialog(QDialog):
     def _onCheckFinished(self, clean):
         if not clean:
             self.graftbtn.setEnabled(False)
-            txt = _('Before graft, you must <a href="commit">'
-                    '<b>commit</b></a> or <a href="discard">'
-                    '<b>discard</b></a> changes.')
+            txt = _('Before graft, you must '
+                    '<a href="commit"><b>commit</b></a>, '
+                    '<a href="shelve"><b>shelve</b></a> to patch, '
+                    'or <a href="discard"><b>discard</b></a> changes.')
         else:
             self.graftbtn.setEnabled(True)
             txt = _('You may continue or start the graft')
@@ -174,8 +199,8 @@ class GraftDialog(QDialog):
 
     def graft(self):
         self.graftbtn.setEnabled(False)
-        self.cancelbtn.setShown(False)
-        opts = dict((n, w.isChecked()) for n, w in self._optchks.iteritems())
+        self.cancelbtn.setVisible(False)
+        opts = dict((n, w.isChecked()) for n, w in self._optchks.items())
         itool = opts.pop('autoresolve') and 'merge' or 'fail'
         opts['config'] = 'ui.merge=internal:%s' % itool
         if os.path.exists(self._graftstatefile):
@@ -197,17 +222,7 @@ class GraftDialog(QDialog):
         sess.commandFinished.connect(self._abortFinished)
 
     def graftstate(self):
-        graftstatefile = self.repo.join('graftstate')
-        if os.path.exists(graftstatefile):
-            f = open(graftstatefile, 'r')
-            info = f.readlines()
-            f.close()
-            if len(info):
-                revlist = [rev.strip() for rev in info]
-                revlist = [rev for rev in revlist if rev != '']
-                if revlist:
-                    return revlist
-        return None
+        return hglib.readgraftstate(self.repo)
 
     def _runCommand(self, cmdline):
         assert self._cmdsession.isFinished()
