@@ -5,17 +5,56 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import
+
 import os
 
-from mercurial import hg, ui, mdiff, similar, patch
+from .qtcore import (
+    QAbstractTableModel,
+    QModelIndex,
+    QSettings,
+    QThread,
+    QTimer,
+    Qt,
+    pyqtSignal,
+)
+from .qtgui import (
+    QAbstractItemView,
+    QCheckBox,
+    QDialog,
+    QFrame,
+    QHBoxLayout,
+    QLabel,
+    QListWidget,
+    QListWidgetItem,
+    QMessageBox,
+    QPushButton,
+    QSizePolicy,
+    QSlider,
+    QSplitter,
+    QTextBrowser,
+    QToolButton,
+    QTreeView,
+    QVBoxLayout,
+)
 
-from tortoisehg.util import hglib, thread2
-from tortoisehg.util.i18n import _
+from mercurial import (
+    hg,
+    patch,
+    pycompat,
+    similar,
+)
 
-from tortoisehg.hgqt import qtlib, htmlui, cmdui
-
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util import (
+    hglib,
+    thread2,
+)
+from ..util.i18n import _
+from . import (
+    cmdui,
+    htmlui,
+    qtlib,
+)
 
 # Techincal debt
 # Try to cut down on the jitter when findRenames is pressed.  May
@@ -34,7 +73,7 @@ class DetectRenameDialog(QDialog):
 
         self.setWindowTitle(_('Detect Copies/Renames in %s')
                             % repoagent.displayName())
-        self.setWindowIcon(qtlib.geticon('detect_rename'))
+        self.setWindowIcon(qtlib.geticon('thg-guess'))
         self.setWindowFlags(Qt.Window)
 
         layout = QVBoxLayout()
@@ -151,10 +190,10 @@ class DetectRenameDialog(QDialog):
         layout.addWidget(self.stbar)
 
         s = QSettings()
-        self.restoreGeometry(s.value('guess/geom').toByteArray())
-        hsplit.restoreState(s.value('guess/hsplit-state').toByteArray())
-        vsplit.restoreState(s.value('guess/vsplit-state').toByteArray())
-        slider.setValue(s.value('guess/simslider').toInt()[0] or 50)
+        self.restoreGeometry(qtlib.readByteArray(s, 'guess/geom'))
+        hsplit.restoreState(qtlib.readByteArray(s, 'guess/hsplit-state'))
+        vsplit.restoreState(qtlib.readByteArray(s, 'guess/vsplit-state'))
+        slider.setValue(qtlib.readInt(s, 'guess/simslider') or 50)
         self.vsplit, self.hsplit = vsplit, hsplit
         QTimer.singleShot(0, self.refresh)
 
@@ -179,7 +218,7 @@ class DetectRenameDialog(QDialog):
             item = QListWidgetItem(hglib.tounicode(x))
             item.orig = x
             self.unrevlist.addItem(item)
-            self.unrevlist.setItemSelected(item, x in self.pats)
+            item.setSelected(x in self.pats)
         if dests:
             self.findbtn.setEnabled(True)
         else:
@@ -220,7 +259,7 @@ class DetectRenameDialog(QDialog):
 
     def searchfinished(self):
         self.stbar.clearProgress()
-        for col in xrange(3):
+        for col in pycompat.xrange(3):
             self.matchtv.resizeColumnToContents(col)
         self.findbtn.setEnabled(self.unrevlist.count())
         self.matchbtn.setEnabled(len(self.matchtv.model().rows))
@@ -250,7 +289,7 @@ class DetectRenameDialog(QDialog):
                       'destination file:\n%s. Aborting!') % udest)
                 return
             remdests[dest] = src
-        for dest, src in remdests.iteritems():
+        for dest, src in remdests.items():
             if not os.path.exists(self.repo.wjoin(src)):
                 wctx.forget([src]) # !->R
             wctx.copy(src, dest)
@@ -265,13 +304,13 @@ class DetectRenameDialog(QDialog):
             return
         index = indexes[0]
         ctx = self.repo['.']
-        hu = htmlui.htmlui()
+        hu = htmlui.htmlui(self.repo.ui)
         row = self.matchtv.model().getRow(index)
         src, dest, percent = self.matchtv.model().getRow(index)
         aa = self.repo.wread(dest)
         rr = ctx.filectx(src).data()
         date = hglib.displaytime(ctx.date())
-        difftext = mdiff.unidiff(rr, date, aa, date, src, dest)
+        difftext = hglib.unidifftext(rr, date, aa, date, src, dest)
         if not difftext:
             t = _('%s and %s have identical contents\n\n') % \
                     (hglib.tounicode(src), hglib.tounicode(dest))
@@ -282,7 +321,7 @@ class DetectRenameDialog(QDialog):
         self.difftb.setHtml(hu.getdata()[0])
 
     def onUnrevDoubleClicked(self, index):
-        file = hglib.fromunicode(self.unrevlist.model().data(index).toString())
+        file = hglib.fromunicode(self.unrevlist.model().data(index))
         qtlib.editfiles(self.repo, [file])
 
     def accept(self):
@@ -326,11 +365,11 @@ class MatchModel(QAbstractTableModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return QVariant()
+            return None
         if role == Qt.DisplayRole:
             s = self.rows[index.row()][index.column()]
             f = self.displayformats[index.column()]
-            return QVariant(f(s))
+            return f(s)
         '''
         elif role == Qt.TextColorRole:
             src, dst, pct = self.rows[index.row()]
@@ -341,13 +380,13 @@ class MatchModel(QAbstractTableModel):
         elif role == Qt.ToolTipRole:
             # explain what row means?
         '''
-        return QVariant()
+        return None
 
     def headerData(self, col, orientation, role):
         if role != Qt.DisplayRole or orientation != Qt.Horizontal:
-            return QVariant()
+            return None
         else:
-            return QVariant(self.headers[col])
+            return self.headers[col]
 
     def flags(self, index):
         return Qt.ItemIsSelectable | Qt.ItemIsEnabled
@@ -382,11 +421,12 @@ class MatchModel(QAbstractTableModel):
         self.layoutChanged.emit()
 
     def sort(self, col, order):
+        self.beginResetModel()
         self.layoutAboutToBeChanged.emit()
         self.rows.sort(key=lambda x: x[col],
                        reverse=(order == Qt.DescendingOrder))
         self.layoutChanged.emit()
-        self.reset()
+        self.endResetModel()
 
     def isEmpty(self):
         return not bool(self.rows)
@@ -394,12 +434,12 @@ class MatchModel(QAbstractTableModel):
 class RenameSearchThread(QThread):
     '''Background thread for searching repository history'''
     match = pyqtSignal(object)
-    progress = pyqtSignal(QString, object, QString, QString, object)
-    showMessage = pyqtSignal(unicode)
+    progress = pyqtSignal(str, object, str, str, object)
+    showMessage = pyqtSignal(str)
 
     def __init__(self, repo, ufiles, minpct, copies):
         super(RenameSearchThread, self).__init__()
-        self.repo = hg.repository(ui.ui(), repo.root)
+        self.repo = hg.repository(hglib.loadui(), repo.root)
         self.ufiles = ufiles
         self.minpct = minpct
         self.copies = copies
@@ -414,12 +454,11 @@ class RenameSearchThread(QThread):
         self.repo.ui.progress = emit
         self.threadid = int(self.currentThreadId())
         try:
-            try:
-                self.search(self.repo)
-            except KeyboardInterrupt:
-                pass
-            except Exception, e:
-                self.showMessage.emit(hglib.tounicode(str(e)))
+            self.search(self.repo)
+        except KeyboardInterrupt:
+            pass
+        except Exception as e:
+            self.showMessage.emit(hglib.tounicode(str(e)))
         finally:
             self.threadid = None
 
@@ -442,11 +481,11 @@ class RenameSearchThread(QThread):
         else:
             ws = wctx.status()
             srcs = ws.removed + ws.deleted
-        added = [wctx[a] for a in self.ufiles]
-        removed = [pctx[a] for a in srcs if a in pctx]
+        added = [wctx[a] for a in sorted(self.ufiles)]
+        removed = [pctx[a] for a in sorted(srcs) if a in pctx]
         # do not consider files of zero length
-        added = sorted([fctx for fctx in added if fctx.size() > 0])
-        removed = sorted([fctx for fctx in removed if fctx.size() > 0])
+        added = [fctx for fctx in added if fctx.size() > 0]
+        removed = [fctx for fctx in removed if fctx.size() > 0]
         exacts = []
         gen = similar._findexactmatches(repo, added, removed)
         for o, n in gen:
