@@ -5,17 +5,63 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-import os, re
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+import os
+import re
 
-from mercurial import error
+from .qtcore import (
+    QAbstractListModel,
+    QByteArray,
+    QMimeData,
+    QModelIndex,
+    QObject,
+    QPoint,
+    QTimer,
+    QUrl,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAbstractItemView,
+    QAction,
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QDockWidget,
+    QFont,
+    QFrame,
+    QHBoxLayout,
+    QInputDialog,
+    QListView,
+    QMenu,
+    QMessageBox,
+    QPushButton,
+    QToolBar,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
+)
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import cmdcore, qtlib, cmdui
-from tortoisehg.hgqt import commit, qdelete, qfold, qrename, rejects
+from mercurial import (
+    error,
+    pycompat,
+    scmutil,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    commit,
+    qtlib,
+    qdelete,
+    qfold,
+    rejects,
+)
 
 def _checkForRejects(repo, rawoutput, parent=None):
     """Parse output of qpush/qpop to resolve hunk failure manually"""
@@ -34,7 +80,7 @@ def _checkForRejects(repo, rawoutput, parent=None):
             rejfiles[wfile] = (r == QDialog.Accepted)
 
     # empty rejfiles means we failed to parse output message
-    return bool(rejfiles) and all(rejfiles.itervalues())
+    return bool(rejfiles) and all(rejfiles.values())
 
 class QueueManagementActions(QObject):
     """Container for patch queue management actions"""
@@ -52,7 +98,7 @@ class QueueManagementActions(QObject):
             'deleteQueue': QAction(_('&Delete Queue...'), self),
             'purgeQueue':  QAction(_('&Purge Queue...'), self),
             }
-        for name, action in self._actions.iteritems():
+        for name, action in self._actions.items():
             action.triggered.connect(getattr(self, '_' + name))
         self._updateActions()
 
@@ -62,7 +108,7 @@ class QueueManagementActions(QObject):
 
     def _updateActions(self):
         enabled = bool(self._repoagent) and self._cmdsession.isFinished()
-        for action in self._actions.itervalues():
+        for action in self._actions.values():
             action.setEnabled(enabled)
 
     def createMenu(self, parent=None):
@@ -275,7 +321,7 @@ class PatchQueueActions(QObject):
         return self._runCommand('qdelete', patches, dlg.options())
 
     def foldPatches(self, patches):
-        lpatches = map(hglib.fromunicode, patches)
+        lpatches = pycompat.maplist(hglib.fromunicode, patches)
         dlg = qfold.QFoldDialog(self._repoagent, lpatches, self.parent())
         dlg.finished.connect(dlg.deleteLater)
         if not dlg.exec_():
@@ -283,19 +329,11 @@ class PatchQueueActions(QObject):
         return self._runCommand('qfold', dlg.patches(), dlg.options())
 
     def renamePatch(self, patch):
-        newname = patch
-        while True:
-            newname = self._getNewName(_('Rename Patch'),
-                                       _('Rename patch <b>%s</b> to:') % patch,
-                                       newname, _('Rename'))
-            if not newname or patch == newname:
-                return cmdcore.nullCmdSession()
-            repo = self._repoagent.rawRepo()
-            newfilename = hglib.tounicode(
-                repo.mq.join(hglib.fromunicode(newname)))
-            ok = qrename.checkPatchname(newfilename, self.parent())
-            if ok:
-                break
+        newname = self._getNewName(_('Rename Patch'),
+                                   _('Rename patch <b>%s</b> to:') % patch,
+                                   patch, _('Rename'))
+        if not newname or patch == newname:
+            return cmdcore.nullCmdSession()
         return self._runCommand('qrename', [patch, newname], {})
 
     def guardPatch(self, patch, guards):
@@ -378,15 +416,30 @@ class PatchQueueModel(QAbstractListModel):
 
     def _updateCacheAndLayout(self):
         self.layoutAboutToBeChanged.emit()
+        oldcount = len(self._series)
         oldindexes = [(oi, self._series[oi.row()])
                       for oi in self.persistentIndexList()]
         self._buildCache()
+
+        mappedindexes = []  # old -> new
+        missngindexes = []  # old
         for oi, patch in oldindexes:
             try:
                 ni = self.index(self._series.index(patch), oi.column())
+                mappedindexes.append((oi, ni))
             except ValueError:
-                ni = QModelIndex()
+                missngindexes.append(oi)
+        # if no indexes are moved, assume missing ones were renamed
+        if (missngindexes and oldcount == len(self._series)
+            and all(oi.row() == ni.row() for oi, ni in mappedindexes)):
+            mappedindexes.extend((oi, self.index(oi.row(), oi.column()))
+                                 for oi in missngindexes)
+            del missngindexes[:]
+
+        for oi, ni in mappedindexes:
             self.changePersistentIndex(oi, ni)
+        for oi in missngindexes:
+            self.changePersistentIndex(oi, QModelIndex())
         self.layoutChanged.emit()
 
     def _buildCache(self):
@@ -432,7 +485,7 @@ class PatchQueueModel(QAbstractListModel):
         return len(self._series)
 
     def appliedCount(self):
-        return sum(s == 'applied' for s in self._statusmap.itervalues())
+        return sum(s == 'applied' for s in self._statusmap.values())
 
     def patchName(self, index):
         if not index.isValid():
@@ -442,7 +495,8 @@ class PatchQueueModel(QAbstractListModel):
     def patchGuards(self, index):
         if not index.isValid():
             return []
-        return map(hglib.tounicode, self._seriesguards[index.row()])
+        return pycompat.maplist(hglib.tounicode,
+                                self._seriesguards[index.row()])
 
     def isApplied(self, index):
         if not index.isValid():
@@ -473,7 +527,10 @@ class PatchQueueModel(QAbstractListModel):
         repo = self._repoagent.rawRepo()
         patch = self._series[index.row()]
         try:
-            ctx = repo.changectx(patch)
+            if patch in repo.thgmqunappliedpatches:
+                ctx = repo[patch]
+            else:
+                ctx = scmutil.revsymbol(repo, patch)
         except error.RepoLookupError:
             # cache not updated after qdelete or qfinish
             return
@@ -520,7 +577,7 @@ class PatchQueueModel(QAbstractListModel):
             after = None  # next to working rev
         patches = str(data.data('application/vnd.thg.mq.series')).splitlines()
         cmdline = hglib.buildcmdargs('qreorder', after=after, *patches)
-        cmdline = map(hglib.tounicode, cmdline)
+        cmdline = pycompat.maplist(hglib.tounicode, cmdline)
         self._repoagent.runCommand(cmdline)
         return True
 
@@ -570,13 +627,22 @@ class MQPatchesWidget(QDockWidget):
         a.setToolTip(_('Unapply all patches'))
         self._qgotoAct = QAction(
             qtlib.geticon('hg-qgoto'), _('Go &to Patch'), self)
+        self._qpushMoveAct = a = QAction(
+            qtlib.geticon('hg-qpush-move'), _('&Apply Only This Patch'), self)
+        a.setToolTip(_('Apply only the selected patch'))
+        qtlib.setContextMenuShortcut(a, 'Ctrl+Return')
+        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         self._qfinishAct = a = QAction(
             qtlib.geticon('qfinish'), _('&Finish Patch'), self)
         a.setToolTip(_('Move applied patches into repository history'))
         self._qdeleteAct = a = QAction(
             qtlib.geticon('hg-qdelete'), _('&Delete Patches...'), self)
+        qtlib.setContextMenuShortcut(a, 'Del')
+        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         a.setToolTip(_('Delete selected patches'))
-        self._qrenameAct = QAction(_('Re&name Patch...'), self)
+        self._qrenameAct = a = QAction(_('Re&name Patch...'), self)
+        qtlib.setContextMenuShortcut(a, 'F2')
+        a.setShortcutContext(Qt.WidgetWithChildrenShortcut)
         self._setGuardsAct = a = QAction(
             qtlib.geticon('hg-qguard'), _('Set &Guards...'), self)
         a.setToolTip(_('Configure guards for selected patch'))
@@ -589,9 +655,14 @@ class MQPatchesWidget(QDockWidget):
         tbar.addAction(self._qpopAct)
         tbar.addAction(self._qpopAllAct)
         tbar.addSeparator()
+        tbar.addAction(self._qpushMoveAct)
+        self.addAction(self._qpushMoveAct)
+        tbar.addSeparator()
         tbar.addAction(self._qfinishAct)
         tbar.addAction(self._qdeleteAct)
+        self.addAction(self._qdeleteAct)
         tbar.addSeparator()
+        self.addAction(self._qrenameAct)
         tbar.addAction(self._setGuardsAct)
 
         self._queueFrame = w = QFrame()
@@ -640,11 +711,12 @@ class MQPatchesWidget(QDockWidget):
 
         self._queueListWidget.activated.connect(self._onGotoPatch)
 
-        self._qpushAct.triggered[()].connect(self._patchActions.pushPatch)
+        self._qpushAct.triggered.connect(self._patchActions.pushPatch)
         self._qpushAllAct.triggered.connect(self._patchActions.pushAllPatches)
-        self._qpopAct.triggered[()].connect(self._patchActions.popPatch)
+        self._qpopAct.triggered.connect(self._patchActions.popPatch)
         self._qpopAllAct.triggered.connect(self._patchActions.popAllPatches)
         self._qgotoAct.triggered.connect(self._onGotoPatch)
+        self._qpushMoveAct.triggered.connect(self._onPushMovePatch)
         self._qfinishAct.triggered.connect(self._onFinishRevision)
         self._qdeleteAct.triggered.connect(self._onDelete)
         self._qrenameAct.triggered.connect(self._onRenamePatch)
@@ -699,6 +771,7 @@ class MQPatchesWidget(QDockWidget):
     def _onMenuRequested(self, pos):
         menu = QMenu(self)
         menu.addAction(self._qgotoAct)
+        menu.addAction(self._qpushMoveAct)
         menu.addAction(self._qfinishAct)
         menu.addAction(self._qdeleteAct)
         menu.addAction(self._qrenameAct)
@@ -729,7 +802,7 @@ class MQPatchesWidget(QDockWidget):
     def _onDelete(self):
         model = self._queueListWidget.model()
         selmodel = self._queueListWidget.selectionModel()
-        patches = map(model.patchName, selmodel.selectedRows())
+        patches = pycompat.maplist(model.patchName, selmodel.selectedRows())
         self._patchActions.deletePatches(patches)
 
     @pyqtSlot()
@@ -737,6 +810,11 @@ class MQPatchesWidget(QDockWidget):
         patch = self._currentPatchName()
         self._patchActions.gotoPatch(patch)
 
+    @pyqtSlot()
+    def _onPushMovePatch(self):
+        patch = self._currentPatchName()
+        self._patchActions.pushPatch(patch, move=True)
+
     @pyqtSlot()
     def _onFinishRevision(self):
         patch = self._currentPatchName()
@@ -769,6 +847,7 @@ class MQPatchesWidget(QDockWidget):
         anyapplied = any(model.isApplied(i) for i in indexes)
         self._qgotoAct.setEnabled(len(indexes) == 1
                                   and indexes[0] != model.topAppliedIndex())
+        self._qpushMoveAct.setEnabled(len(indexes) == 1 and not anyapplied)
         self._qfinishAct.setEnabled(len(indexes) == 1 and anyapplied)
         self._qdeleteAct.setEnabled(len(indexes) > 0 and not anyapplied)
         self._setGuardsAct.setEnabled(len(indexes) == 1)
@@ -837,7 +916,8 @@ class MQPatchesWidget(QDockWidget):
             newguards.append(guard)
         elif guard in newguards:
             newguards.remove(guard)
-        self._patchActions.selectGuards(map(hglib.tounicode, newguards))
+        self._patchActions.selectGuards(pycompat.maplist(hglib.tounicode,
+                                                         newguards))
 
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Escape:
@@ -877,7 +957,7 @@ class OptionsDialog(QDialog):
         self.bb = bb
         layout.addWidget(bb)
 
-    @qtlib.senderSafeSlot()
+    @pyqtSlot()
     def _resolveopts(self):
         # cannot use both --force and --keep-changes
         exclmap = {self.forcecb: [self.keepcb],
