@@ -151,7 +151,7 @@ class PhabReviewDialog(QDialog):
             return None
 
         scheme, hostpath = url.split(b'://', 1)
-        if hostpath[-1] == b'/':
+        if hostpath.endswith(b'/'):
             hostpath = hostpath[0:-1]
 
         if not withcallsign:
@@ -297,7 +297,7 @@ class PhabReviewDialog(QDialog):
 
         reviewerlist = self._qui.selected_reviewers_list
         opts['reviewer'] = [reviewerlist.item(i).data(Qt.UserRole + 1).username
-                            for i in xrange(reviewerlist.count())]
+                            for i in pycompat.xrange(reviewerlist.count())]
 
         return opts
 
@@ -373,11 +373,10 @@ class PhabReviewDialog(QDialog):
             url = b'Not Configured!'
 
         reviewerlist = self._qui.selected_reviewers_list
-        tounicode = hglib.tounicode
         role = Qt.UserRole + 1
 
-        reviewers = [tounicode(reviewerlist.item(i).data(role).username)
-                     for i in xrange(reviewerlist.count())]
+        reviewers = [reviewerlist.item(i).data(role).username
+                     for i in pycompat.xrange(reviewerlist.count())]
         if reviewers:
             reviewers = u', '.join(reviewers)
         else:
