@@ -21,22 +21,67 @@ import sys
 import tempfile
 import weakref
 
+from .qtcore import (
+    PYQT_VERSION,
+    QByteArray,
+    QDir,
+    QEvent,
+    QFile,
+    QObject,
+    QProcess,
+    QSize,
+    QUrl,
+    QT_VERSION,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QApplication,
+    QComboBox,
+    QCommonStyle,
+    QColor,
+    QDesktopServices,
+    QDialog,
+    QFont,
+    QFrame,
+    QHBoxLayout,
+    QIcon,
+    QInputDialog,
+    QKeySequence,
+    QLabel,
+    QLineEdit,
+    QMessageBox,
+    QPainter,
+    QPalette,
+    QPixmap,
+    QPushButton,
+    QShortcut,
+    QSizePolicy,
+    QStyle,
+    QStyleOptionButton,
+    QVBoxLayout,
+    QWidget,
+)
+
 from mercurial import (
     color,
     extensions,
+    pycompat,
     util,
 )
+from mercurial.utils import (
+    procutil,
+    stringutil,
+)
 
-from tortoisehg.util import (
+from ..util import (
     editor,
     hglib,
     paths,
     terminal,
 )
-from tortoisehg.util.i18n import _
-
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util.i18n import _
 
 try:
     import win32con
@@ -47,16 +92,6 @@ except ImportError:
 # largest allowed size for widget, defined in <src/gui/kernel/qwidget.h>
 QWIDGETSIZE_MAX = (1 << 24) - 1
 
-if PYQT_VERSION < 0x40704:
-    # QObject.sender() goes wrong if connection made in Qt layer.  consider
-    # using QSignalMapper for zero-argument slots instead.
-    def senderSafeSlot(*types, **opts):
-        def deco(func):
-            return func
-        return deco
-else:
-    senderSafeSlot = pyqtSlot
-
 tmproot = None
 def gettempdir():
     global tmproot
@@ -136,7 +171,7 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
         dlg.exec_()
         return
 
-    files = [util.shellquote(util.localpath(f)) for f in files]
+    files = [procutil.shellquote(util.localpath(f)) for f in files]
     assert len(files) == 1 or lineno == None
 
     cmdline = None
@@ -189,10 +224,10 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
                 expanded.append(phrase)
             expanded.append(toolpath[pos:])
             cmdline = ' '.join(expanded + files)
-        except ValueError, e:
+        except ValueError as e:
             # '[' or ']' not found
             pass
-        except TypeError, e:
+        except TypeError as e:
             # variable expansion failed
             pass
 
@@ -200,21 +235,21 @@ def editfiles(repo, files, lineno=None, search=None, parent=None):
     try:
         if '$FILES' in cmdline:
             cmdline = cmdline.replace('$FILES', ' '.join(files))
-            cmdline = util.quotecommand(cmdline)
+            cmdline = procutil.quotecommand(cmdline)
             subprocess.Popen(cmdline, shell=shell, creationflags=openflags,
                              stderr=None, stdout=None, stdin=None, cwd=cwd)
         elif '$FILE' in cmdline:
             for file in files:
                 cmd = cmdline.replace('$FILE', file)
-                cmd = util.quotecommand(cmd)
+                cmd = procutil.quotecommand(cmd)
                 subprocess.Popen(cmd, shell=shell, creationflags=openflags,
                                  stderr=None, stdout=None, stdin=None, cwd=cwd)
         else:
             # assume filenames were expanded already
-            cmdline = util.quotecommand(cmdline)
+            cmdline = procutil.quotecommand(cmdline)
             subprocess.Popen(cmdline, shell=shell, creationflags=openflags,
                              stderr=None, stdout=None, stdin=None, cwd=cwd)
-    except (OSError, EnvironmentError), e:
+    except (OSError, EnvironmentError) as e:
         QMessageBox.warning(parent,
                 _('Editor launch failure'),
                 u'%s : %s' % (hglib.tounicode(cmdline),
@@ -245,7 +280,7 @@ def openshell(root, reponame, ui=None):
         cwd = os.getcwd()
         try:
             # Unix: QProcess.startDetached(program) cannot parse single-quoted
-            # parameters built using util.shellquote().
+            # parameters built using procutil.shellquote().
             # Windows: subprocess.Popen(program, shell=True) cannot spawn
             # cmd.exe in new window, probably because the initial cmd.exe is
             # invoked with SW_HIDE.
@@ -254,7 +289,8 @@ def openshell(root, reponame, ui=None):
                 # can't parse shellcmd in POSIX way
                 started = QProcess.startDetached(hglib.tounicode(shellcmd))
             else:
-                fullargs = map(hglib.tounicode, shlex.split(shellcmd))
+                fullargs = pycompat.maplist(hglib.tounicode,
+                                            shlex.split(shellcmd))
                 started = QProcess.startDetached(fullargs[0], fullargs[1:])
         finally:
             os.chdir(cwd)
@@ -266,6 +302,62 @@ def openshell(root, reponame, ui=None):
                    _('A terminal shell must be configured'))
 
 
+# 'type' argument of QSettings.value() can't be used because:
+#  a) it appears to be broken before PyQt 4.11.x (#4882)
+#  b) it may raise TypeError if a setting has a value of an unexpected type
+
+def readBool(qs, key, default=False):
+    """Read the specified value from QSettings and coerce into bool"""
+    v = qs.value(key, default)
+    if isinstance(v, basestring):
+        # qvariant.cpp:qt_convertToBool()
+        return not (v == '0' or v == 'false' or v == '')
+    return bool(v)
+
+def readByteArray(qs, key, default=b''):
+    """Read the specified value from QSettings and coerce into QByteArray"""
+    v = qs.value(key, default)
+    if v is None:
+        return QByteArray(default)
+    try:
+        return QByteArray(v)
+    except TypeError:
+        return QByteArray(default)
+
+def readInt(qs, key, default=0):
+    """Read the specified value from QSettings and coerce into int"""
+    v = qs.value(key, default)
+    if v is None:
+        return int(default)
+    try:
+        return int(v)
+    except (TypeError, ValueError):
+        return int(default)
+
+def readString(qs, key, default=''):
+    """Read the specified value from QSettings and coerce into string"""
+    v = qs.value(key, default)
+    if v is None:
+        return unicode(default)
+    try:
+        return unicode(v)
+    except ValueError:
+        return unicode(default)
+
+def readStringList(qs, key, default=()):
+    """Read the specified value from QSettings and coerce into string list"""
+    v = qs.value(key, default)
+    if v is None:
+        return list(default)
+    if isinstance(v, basestring):
+        # qvariant.cpp:convert()
+        return [v]
+    try:
+        return [unicode(e) for e in v]
+    except (TypeError, ValueError):
+        return list(default)
+
+
 def isDarkTheme(palette=None):
     """True if white-on-black color scheme is preferable"""
     if not palette:
@@ -393,7 +485,7 @@ def markup(msg, **styles):
         style[name] = value
     style = ';'.join(['%s: %s' % t for t in style.items()])
     msg = hglib.tounicode(msg)
-    msg = Qt.escape(msg)
+    msg = cgi.escape(msg)
     msg = msg.replace('\n', '<br />')
     return u'<span style="%s">%s</span>' % (style, msg)
 
@@ -607,7 +699,7 @@ def getcheckboxpixmap(state, bgcolor, widget):
     option = QStyleOptionButton()
     style = QApplication.style()
     option.initFrom(widget)
-    option.rect = style.subElementRect(style.SE_CheckBoxIndicator, option)
+    option.rect = style.subElementRect(style.SE_CheckBoxIndicator, option, None)
     option.rect.moveTo(1, 1)
     option.state |= state
     style.drawPrimitive(style.PE_IndicatorCheckBox, option, painter)
@@ -623,7 +715,7 @@ def getcheckboxpixmap(state, bgcolor, widget):
 # icons to be rendered unusably small. The workaround for that is to render
 # the icons at double the normal size.
 # TODO: Remove this hack after upgrading to Qt5.
-IS_RETINA = util.parsebool(os.environ.get('THG_RETINA', '0'))
+IS_RETINA = stringutil.parsebool(os.environ.get('THG_RETINA', '0'))
 
 def _fixIconSizeForRetinaDisplay(s):
     if IS_RETINA:
@@ -675,7 +767,6 @@ class ThgFont(QObject):
 _fontdefaults = {
     'fontcomment': 'monospace,10',
     'fontdiff': 'monospace,10',
-    'fontlist': 'sans,9',
     'fontlog': 'monospace,10',
     'fontoutputlog': 'sans,8'
 }
@@ -699,7 +790,7 @@ def CommonMsgBox(icon, title, main, text='', buttons=QMessageBox.Ok,
     msg.setWindowTitle(title)
     msg.setStandardButtons(buttons)
     for button_id, label in labels:
-        msg.setButtonText(button_id, label)
+        msg.button(button_id).setText(label)
     if defaultbutton:
         msg.setDefaultButton(defaultbutton)
     msg.setText('<b>%s</b>' % main)
@@ -749,7 +840,7 @@ class CustomPrompt(QMessageBox):
         return self.exec_()
 
     def keyPressEvent(self, event):
-        for k, btn in self.hotkeys.iteritems():
+        for k, btn in self.hotkeys.items():
             if event.text() == k:
                 btn.clicked.emit(False)
         super(CustomPrompt, self).keyPressEvent(event)
@@ -816,6 +907,7 @@ class BadCompletionBlocker(QObject):
     replaces the edit text by the current completion on enter key pressed.
     This is wrong in the following scenario:
 
+    >>> from .qtgui import QKeyEvent
     >>> combo = QComboBox(editable=True)
     >>> combo.addItem('history value')
     >>> combo.setEditText('initial value')
@@ -918,7 +1010,7 @@ class PMButton(QPushButton):
         self.setIcon(icon)
 
     def is_expanded(self):
-        return self.icon().serialNumber() == self.minus.serialNumber()
+        return self.icon().cacheKey() == self.minus.cacheKey()
 
     def is_collapsed(self):
         return not self.is_expanded()
@@ -1012,7 +1104,7 @@ class StatusLabel(QWidget):
             elif isinstance(icon, basestring):
                 icon = geticon(icon)
             elif not isinstance(icon, QIcon):
-                raise TypeError, '%s: bool, str or QIcon' % type(icon)
+                raise TypeError('%s: bool, str or QIcon' % type(icon))
             self.status_icon.setVisible(True)
             self.status_icon.setPixmap(icon.pixmap(16, 16))
 
@@ -1102,6 +1194,7 @@ class DialogKeeper(QObject):
 
     closed dialog will be deleted:
 
+    >>> from .qtcore import QEventLoop, QTimer
     >>> def processDeferredDeletion():
     ...     loop = QEventLoop()
     ...     QTimer.singleShot(0, loop.quit)
@@ -1195,7 +1288,7 @@ class DialogKeeper(QObject):
                 del self._keytodlgs[key]
 
     def count(self):
-        return sum(len(dlgs) for dlgs in self._keytodlgs.itervalues())
+        return sum(len(dlgs) for dlgs in self._keytodlgs.values())
 
     @staticmethod
     def _defaultgenkey(_parent, *args, **_kwargs):
@@ -1376,3 +1469,9 @@ class PaletteSwitcher(object):
         else:
             pl = self._defaultpalette
         targetwidget.setPalette(pl)
+
+def setContextMenuShortcut(action, shortcut):
+    """Set shortcut for a context menu action, making sure it's visible"""
+    action.setShortcut(shortcut)
+    if QT_VERSION >= 0x50a00 and PYQT_VERSION >= 0x50a00:
+        action.setShortcutVisibleInContextMenu(True)
