@@ -5,23 +5,53 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, sys
+from __future__ import absolute_import
 
-from mercurial import util
+import os
+import sys
 
-from tortoisehg.util import hglib, shlib
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, status, cmdcore, cmdui, lfprompt
+from .qtcore import (
+    QObject,
+    QSettings,
+    Qt,
+)
+from .qtgui import (
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QKeySequence,
+    QLabel,
+    QHBoxLayout,
+    QPushButton,
+    QShortcut,
+    QVBoxLayout,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from mercurial import (
+    error,
+    pycompat,
+    util,
+)
+
+from ..util import (
+    hglib,
+    shlib,
+)
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    lfprompt,
+    qtlib,
+    status,
+)
 
 LABELS = { 'add': (_('Checkmark files to add'), _('Add')),
            'forget': (_('Checkmark files to forget'), _('Forget')),
            'revert': (_('Checkmark files to revert'), _('Revert')),
            'remove': (_('Checkmark files to remove'), _('Remove')),}
 
-ICONS = { 'add': 'fileadd',
+ICONS = { 'add': 'hg-add',
            'forget': 'hg-remove',
            'revert': 'hg-revert',
            'remove': 'hg-remove',}
@@ -46,7 +76,7 @@ class QuickOpDialog(QDialog):
         self.setWindowIcon(qtlib.geticon(ICONS[command]))
 
         layout = QVBoxLayout()
-        layout.setMargin(0)
+        layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
 
         toplayout = QVBoxLayout()
@@ -77,7 +107,7 @@ class QuickOpDialog(QDialog):
         defcheck = checktypes[self.command]
 
         opts = {}
-        for s, val in status.statusTypes.iteritems():
+        for s, val in status.statusTypes.items():
             opts[val.name] = s in filetypes
 
         opts['checkall'] = True # pre-check all matching files
@@ -125,13 +155,14 @@ class QuickOpDialog(QDialog):
 
         s = QSettings()
         stwidget.loadSettings(s, 'quickop')
-        self.restoreGeometry(s.value('quickop/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'quickop/geom'))
         if hasattr(self, 'chk'):
             if self.command == 'revert':
-                self.chk.setChecked(s.value('quickop/nobackup', True).toBool())
+                self.chk.setChecked(
+                    qtlib.readBool(s, 'quickop/nobackup', True))
             elif self.command == 'remove':
                 self.chk.setChecked(
-                    s.value('quickop/forceremove', False).toBool())
+                    qtlib.readBool(s, 'quickop/forceremove', False))
         self.stwidget = stwidget
         self.stwidget.refreshWctx()
         QShortcut(QKeySequence('Ctrl+Return'), self, self.accept)
@@ -155,7 +186,7 @@ class QuickOpDialog(QDialog):
             cmdlines.append(hglib.buildcmdargs(self.command, *lfiles, **lopts))
         self.files = files + lfiles
 
-        ucmdlines = [map(hglib.tounicode, xs) for xs in cmdlines]
+        ucmdlines = [pycompat.maplist(hglib.tounicode, xs) for xs in cmdlines]
         self._cmdsession = sess = self._repoagent.runCommandSequence(ucmdlines,
                                                                      self)
         sess.commandFinished.connect(self.commandFinished)
@@ -185,12 +216,16 @@ class QuickOpDialog(QDialog):
                                 _('No operation to perform'),
                                 parent=self)
             return
-        self.repo.bfstatus = True
-        self.repo.lfstatus = True
-        repostate = self.repo.status()
-        self.repo.bfstatus = False
-        self.repo.lfstatus = False
         if self.command == 'remove':
+            self.repo.lfstatus = True
+            try:
+                repostate = self.repo.status()
+            except (EnvironmentError, error.Abort) as e:
+                qtlib.WarningMsgBox(_('Unable to read repository status'),
+                                    hglib.tounicode(str(e)), parent=self)
+                return
+            finally:
+                self.repo.lfstatus = False
             if not self.chk.isChecked():
                 modified = repostate[0]
                 selmodified = []
@@ -267,7 +302,7 @@ class HeadlessQuickop(QObject):
         QObject.__init__(self)
         self.files = cmdline[1:]
         self._cmddialog = cmdui.CmdSessionDialog()
-        sess = repoagent.runCommand(map(hglib.tounicode, cmdline))
+        sess = repoagent.runCommand(pycompat.maplist(hglib.tounicode, cmdline))
         sess.commandFinished.connect(self.commandFinished)
         self._cmddialog.setSession(sess)
 
