@@ -22,7 +22,6 @@ from .qtgui import (
     QComboBox,
     QCompleter,
     QFontMetrics,
-    QIcon,
     QLabel,
     QLineEdit,
     QMainWindow,
@@ -35,7 +34,9 @@ from .qtgui import (
 
 from mercurial import (
     error,
+    pycompat,
     repoview,
+    scmutil,
     util,
 )
 
@@ -61,6 +62,9 @@ def _firstword(query):
 
 def _querytype(repo, query):
     r"""
+    >>> # TODO: maybe replace with real repo
+    >>> origisrevsymbol = scmutil.isrevsymbol
+    >>> scmutil.isrevsymbol = lambda repo, changeid: changeid in repo
     >>> repo = set('0 1 2 3 . stable'.split())
     >>> _querytype(repo, u'') is None
     True
@@ -80,6 +84,7 @@ def _querytype(repo, query):
     'revset'
     >>> _querytype(repo, u'\u3000')  # UnicodeEncodeError
     'revset'
+    >>> scmutil.isrevsymbol = origisrevsymbol
     """
     if not query:
         return
@@ -92,7 +97,7 @@ def _querytype(repo, query):
     if not changeid:
         return 'keyword'
     try:
-        if changeid in repo:
+        if scmutil.isrevsymbol(repo, changeid):
             return 'revset'
     except error.LookupError:  # ambiguous changeid
         pass
@@ -420,6 +425,8 @@ class RepoFilterBar(QToolBar):
         self._branchCombo.setMaxVisibleItems(30)
         self._branchCombo.currentIndexChanged.connect(self._emitBranchChanged)
         completer = QCompleter(self._branchCombo.model(), self._branchCombo)
+        if hasattr(completer, 'setFilterMode'):  # Qt>=5.2
+            completer.setFilterMode(Qt.MatchContains)
         self._branchCombo.setCompleter(completer)
 
         self.addWidget(self._branchLabel)
@@ -463,7 +470,7 @@ class RepoFilterBar(QToolBar):
         priomap = {self._repo.dirstate.branch(): -2, 'default': -1}
         branches = sorted(branches, key=lambda e: priomap.get(e, 0))
 
-        branches = map(hglib.tounicode, branches)
+        branches = pycompat.maplist(hglib.tounicode, branches)
 
         self._branchCombo.blockSignals(True)
         self._branchCombo.clear()
