@@ -5,34 +5,34 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import repotreeitem
-
-from PyQt4.QtCore import *
-from PyQt4.QtGui import QFont
+from __future__ import absolute_import, print_function
 
 import os
 
-if PYQT_VERSION < 0x40700:
-    class LocalQXmlStreamReader(QXmlStreamReader):
-        def readNextStartElement(self):
-            while self.readNext() != QXmlStreamReader.Invalid:
-                if self.isEndElement():
-                    return False
-                elif self.isStartElement():
-                    return True
-            return False
+from .qtcore import (
+    QAbstractItemModel,
+    QByteArray,
+    QFile,
+    QIODevice,
+    QMimeData,
+    QModelIndex,
+    QUrl,
+    QXmlStreamReader,
+    QXmlStreamWriter,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QFont,
+)
 
-        def skipCurrentElement(self):
-            depth = 1
-            while depth > 0 and self.readNext() != QXmlStreamReader.Invalid:
-                if self.isEndElement():
-                    depth -= 1
-                elif self.isStartElement():
-                    depth += 1
+from mercurial import (
+    pycompat,
+)
 
-    QXmlStreamReader = LocalQXmlStreamReader
+from ..util import hglib
+from ..util.i18n import _
+from . import repotreeitem
 
 extractXmlElementName = 'reporegextract'
 reporegistryXmlElementName = 'reporegistry'
@@ -55,18 +55,18 @@ def readXml(source, rootElementName):
     itemread = None
     xr = QXmlStreamReader(source)
     if xr.readNextStartElement():
-        ele = str(xr.name().toString())
+        ele = str(xr.name())
         if ele != rootElementName:
-            print "unexpected xml element '%s' "\
-                  "(was looking for %s)" % (ele, rootElementName)
+            print("unexpected xml element '%s' "
+                  "(was looking for %s)" % (ele, rootElementName))
             return
     if xr.hasError():
-        print hglib.fromunicode(xr.errorString(), 'replace')
+        print(hglib.fromunicode(xr.errorString(), 'replace'))
     if xr.readNextStartElement():
         itemread = repotreeitem.undumpObject(xr)
         xr.skipCurrentElement()
     if xr.hasError():
-        print hglib.fromunicode(xr.errorString(), 'replace')
+        print(hglib.fromunicode(xr.errorString(), 'replace'))
     return itemread
 
 def iterRepoItemFromXml(source):
@@ -121,7 +121,7 @@ class RepoTreeModel(QAbstractItemModel):
         self.allrepos = all
         self.updateCommonPaths()
 
-    # see http://doc.qt.nokia.com/4.6/model-view-model-subclassing.html
+    # see https://doc.qt.io/qt-4.8/model-view-programming.html#model-subclassing-reference
 
     # overrides from QAbstractItemModel
 
@@ -164,10 +164,10 @@ class RepoTreeModel(QAbstractItemModel):
 
     def data(self, index, role=Qt.DisplayRole):
         if not index.isValid():
-            return QVariant()
+            return None
         if role not in (Qt.DisplayRole, Qt.EditRole, Qt.DecorationRole,
                 Qt.FontRole):
-            return QVariant()
+            return None
         item = index.internalPointer()
         if role == Qt.FontRole and item is self._activeRepoItem:
             font = QFont()
@@ -181,7 +181,7 @@ class RepoTreeModel(QAbstractItemModel):
             if orientation == Qt.Horizontal:
                 if section == 1:
                     return _('Path')
-        return QVariant()
+        return None
 
     def flags(self, index):
         if not index.isValid():
@@ -216,7 +216,7 @@ class RepoTreeModel(QAbstractItemModel):
         d = QMimeData()
         d.setData(repoRegMimeType, buf)
         if isinstance(item, repotreeitem.RepoItem):
-            d.setUrls([QUrl.fromLocalFile(hglib.tounicode(item.rootpath()))])
+            d.setUrls([QUrl.fromLocalFile(item.rootpath())])
         else:
             d.setText(item.name)
         return d
@@ -257,8 +257,7 @@ class RepoTreeModel(QAbstractItemModel):
     def setData(self, index, value, role=Qt.EditRole):
         if not index.isValid() or role != Qt.EditRole:
             return False
-        s = value.toString()
-        if s.isEmpty():
+        if not value:
             return False
         item = index.internalPointer()
         if item.setData(index.column(), value):
@@ -276,19 +275,19 @@ class RepoTreeModel(QAbstractItemModel):
             row = rgi.childCount()
 
         # make sure all paths are properly normalized
-        root = os.path.normpath(hglib.fromunicode(uroot))
+        uroot = os.path.normpath(uroot)
 
         # Check whether the repo that we are adding is a subrepo
-        knownitem = self.getRepoItem(root, lookForSubrepos=True)
+        knownitem = self.getRepoItem(uroot, lookForSubrepos=True)
         itemIsSubrepo = isinstance(knownitem,
                                    (repotreeitem.StandaloneSubrepoItem,
                                     repotreeitem.SubrepoItem))
 
         self.beginInsertRows(parent, row, row)
         if itemIsSubrepo:
-            ri = repotreeitem.StandaloneSubrepoItem(root)
+            ri = repotreeitem.StandaloneSubrepoItem(uroot)
         else:
-            ri = repotreeitem.RepoItem(root)
+            ri = repotreeitem.RepoItem(uroot)
         rgi.insertChild(row, ri)
         self.endInsertRows()
 
@@ -303,8 +302,7 @@ class RepoTreeModel(QAbstractItemModel):
                 return e
 
     def indexFromRepoRoot(self, uroot, column=0, standalone=False):
-        item = self.getRepoItem(hglib.fromunicode(uroot),
-                                lookForSubrepos=not standalone)
+        item = self.getRepoItem(uroot, lookForSubrepos=not standalone)
         return self._indexFromItem(item, column)
 
     def isKnownRepoRoot(self, uroot, standalone=False):
@@ -324,7 +322,7 @@ class RepoTreeModel(QAbstractItemModel):
         item = index.internalPointer()
         if not isinstance(item, repotreeitem.RepoItem):
             return
-        return hglib.tounicode(item.rootpath())
+        return item.rootpath()
 
     def addGroup(self, name):
         ri = self.rootItem
@@ -396,10 +394,10 @@ class RepoTreeModel(QAbstractItemModel):
             item._sharedpath = tmpitem._sharedpath
             item._valid = tmpitem._valid
             self._emitItemDataChanged(item)
-        return map(hglib.tounicode, invalidpaths)
+        return pycompat.maplist(hglib.tounicode, invalidpaths)
 
     def updateCommonPaths(self, showShortPaths=None):
-        if not showShortPaths is None:
+        if showShortPaths is not None:
             self.showShortPaths = showShortPaths
         for grp in self.rootItem.childs:
             if isinstance(grp, repotreeitem.RepoGroupItem):
@@ -410,16 +408,18 @@ class RepoTreeModel(QAbstractItemModel):
 
     @pyqtSlot(str)
     def _updateShortName(self, uroot):
+        uroot = unicode(uroot)
         repoagent = self._repomanager.repoAgent(uroot)
-        it = self.getRepoItem(hglib.fromunicode(uroot))
+        it = self.getRepoItem(uroot)
         if it:
             it.setShortName(repoagent.shortName())
             self._emitItemDataChanged(it)
 
     @pyqtSlot(str)
     def _updateBaseNode(self, uroot):
+        uroot = unicode(uroot)
         repo = self._repomanager.repoAgent(uroot).rawRepo()
-        it = self.getRepoItem(hglib.fromunicode(uroot))
+        it = self.getRepoItem(uroot)
         if it:
             it.setBaseNode(hglib.repoidnode(repo))
 
