@@ -36,6 +36,10 @@ from .qtgui import (
     QVBoxLayout,
 )
 
+from mercurial import (
+    pycompat,
+)
+
 from ..util import hglib
 from ..util.i18n import _
 from . import (
@@ -89,7 +93,7 @@ class ResolveDialog(QDialog):
             localrevtitle = qtlib.LabeledSeparator(_('Local revision '
                                                      'information'))
             localrevinfo = csinfo.create(repo)
-            localrevinfo.update(repo[None].p1())
+            localrevinfo.update(repo[repo[None].p1().rev()])
             vbox.addWidget(localrevtitle)
             vbox.addWidget(localrevinfo)
             vbox.addStretch()
@@ -100,7 +104,7 @@ class ResolveDialog(QDialog):
             otherrevtitle = qtlib.LabeledSeparator(_('Other revision '
                                                      'information'))
             otherrevinfo = csinfo.create(repo)
-            otherrevinfo.update(repo[None].p2())
+            otherrevinfo.update(repo[repo[None].p2().rev()])
 
             vbox.addWidget(otherrevtitle)
             vbox.addWidget(otherrevinfo)
@@ -263,7 +267,7 @@ class ResolveDialog(QDialog):
             for root, wfile in selected:
                 if root == curroot:
                     cmd.append(os.path.normpath(os.path.join(root, wfile)))
-            cmdlines.append(map(hglib.tounicode, cmd))
+            cmdlines.append(pycompat.maplist(hglib.tounicode, cmd))
             selected = [(r, w) for r, w in selected if r != curroot]
         if cmdlines:
             sess = self._repoagent.runCommandSequence(cmdlines, self)
