@@ -6,26 +6,34 @@
 # This software may be used and distributed according to the terms
 # of the GNU General Public License, incorporated herein by reference.
 
+from __future__ import absolute_import
 
-from mercurial import error
+import cgi
 
-from tortoisehg.util import hglib, obsoleteutil
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import csinfo, qtlib
+from mercurial import (
+    cmdutil,
+    error,
+    scmutil,
+)
 
-from PyQt4.QtCore import *
+from ..util import hglib, obsoleteutil
+from ..util.i18n import _
+from . import (
+    csinfo,
+    qtlib,
+)
 
 def label_func(widget, item, ctx):
     if item == 'cset':
-        if type(ctx.rev()) is str:
+        if isinstance(ctx.rev(), str):
             return _('Patch:')
         return _('Changeset:')
     elif item == 'parents':
         return _('Parent:')
     elif item == 'children':
         return _('Child:')
-    elif item == 'precursors':
-        return _('Precursors:')
+    elif item == 'predecessors':
+        return _('Predecessors:')
     elif item == 'successors':
         return _('Successors:')
     raise csinfo.UnknownItem(item)
@@ -79,7 +87,7 @@ def data_func(widget, item, ctx):
         if not ts:
             return None
         try:
-            tctx = ctx._repo[ts]
+            tctx = scmutil.revsymbol(ctx.repo(), ts)
             return revline_data(tctx)
         except (error.LookupError, error.RepoLookupError, error.RepoError):
             return ts
@@ -92,8 +100,8 @@ def data_func(widget, item, ctx):
         if ctx.rev() is None:
             ctx = ctx.p1()
         return ctx.extra().get('close') is not None
-    elif item == 'precursors':
-        ctxlist = obsoleteutil.first_known_precursors(ctx)
+    elif item == 'predecessors':
+        ctxlist = obsoleteutil.first_known_predecessors(ctx)
         return format_ctxlist(ctxlist)
     elif item == 'successors':
         ctxlist = obsoleteutil.first_known_successors(ctx)
@@ -102,22 +110,37 @@ def data_func(widget, item, ctx):
     raise csinfo.UnknownItem(item)
 
 def create_markup_func(ui):
-    def link_markup(revnum, revid, linkpattern=None):
+    def link_markup(revnum, revid, linkpattern=None, ctx=None):
         mrevid = revid_markup('%s (%s)' % (revnum, revid))
         if linkpattern is None:
             return mrevid
-        link = linkpattern.replace('{node|short}', revid).replace('{rev}', revnum)
-        return '<a href="%s">%s</a>' % (link, mrevid)
+
+        if linkpattern == 'cset:{node|short}':
+            # this is the linkpattern for thg internal hyperlinks
+            href = 'cset:%s' % revid
+        else:
+            if ctx is None:
+                return mrevid
+            try:
+                # evaluates a generic mercurial template for changeset.link
+                href = cmdutil.rendertemplate(ctx, linkpattern)
+            except (error.Abort, error.ParseError):
+                return mrevid
+
+        return '<a href="%s">%s</a>' % (cgi.escape(href, True), mrevid)
+
     def revline_markup(revnum, revid, summary, highlight=None,
-                       branch=None, linkpattern='cset:{node|short}'):
+                       branch=None, linkpattern='cset:{node|short}', ctx=None):
         def branch_markup(branch):
             opts = dict(fg='black', bg='#aaffaa')
+
             return qtlib.markup(' %s ' % branch, **opts)
+
         summary = qtlib.markup(summary)
         if branch:
             branch = branch_markup(branch)
         if revid:
-            rev = link_markup(revnum, revid, linkpattern=linkpattern)
+            rev = link_markup(revnum, revid, linkpattern=linkpattern, ctx=ctx)
             if branch:
                 return '%s %s %s' % (rev, branch, summary)
             return '%s %s' % (rev, summary)
@@ -126,6 +149,7 @@ def create_markup_func(ui):
             if branch:
                 return '%s - %s %s' % (revnum, branch, summary)
             return '%s - %s' % (revnum, summary)
+
     def markup_func(widget, item, value):
         if item in ('cset', 'graft', 'transplant', 'mqoriginalparent',
                     'p4', 'svn', 'converted'):
@@ -135,8 +159,8 @@ def create_markup_func(ui):
                 linkpattern = 'cset:{node|short}'
             if isinstance(value, basestring):
                 return revid_markup(value)
-            return revline_markup(linkpattern=linkpattern, *value)
-        elif item in ('parents', 'children', 'precursors', 'successors'):
+            return revline_markup(linkpattern=linkpattern, *value, ctx=widget.ctx)
+        elif item in ('parents', 'children', 'predecessors', 'successors'):
             csets = []
             for cset in value:
                 if isinstance(cset, basestring):
@@ -145,16 +169,18 @@ def create_markup_func(ui):
                     csets.append(revline_markup(*cset))
             return csets
         raise csinfo.UnknownItem(item)
+
     return markup_func
 
 def RevPanelWidget(repo):
     '''creates a rev panel widget and returns it'''
     custom = csinfo.custom(data=data_func, label=label_func,
                            markup=create_markup_func(repo.ui))
-    style = csinfo.panelstyle(contents=('cset', 'branch', 'obsolete', 'close', 'user',
+    style = csinfo.panelstyle(contents=(
+                   'cset', 'gitcommit', 'branch', 'obsolete', 'close', 'user',
                    'dateage', 'parents', 'children', 'tags', 'graft', 'transplant',
                    'mqoriginalparent',
-                   'precursors', 'successors',
+                   'predecessors', 'successors',
                    'p4', 'svn', 'converted'), selectable=True,
                    expandable=True)
     return csinfo.create(repo, style=style, custom=custom)
