@@ -5,20 +5,80 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import, print_function
+
 import os
 
-from mercurial import ui, util, error, extensions, scmutil, phases
+from .qtcore import (
+    QEvent,
+    QSettings,
+    QTimer,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QComboBox,
+    QCompleter,
+    QDialog,
+    QDialogButtonBox,
+    QDirModel,
+    QFileDialog,
+    QFont,
+    QFontDialog,
+    QFormLayout,
+    QFrame,
+    QGridLayout,
+    QHBoxLayout,
+    QIcon,
+    QIntValidator,
+    QLabel,
+    QLineEdit,
+    QListView,
+    QListWidget,
+    QListWidgetItem,
+    QMessageBox,
+    QPalette,
+    QPushButton,
+    QRadioButton,
+    QStackedWidget,
+    QStyle,
+    QTabWidget,
+    QTextBrowser,
+    QTextEdit,
+    QVBoxLayout,
+    QWidget,
+)
 
-from tortoisehg.util import hglib, paths, wconfig, i18n, editor
-from tortoisehg.util import terminal, gpg
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib, qscilib, thgrepo, customtools, fileencoding
+from mercurial import (
+    error,
+    extensions,
+    phases,
+    pycompat,
+    util,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util import (
+    editor,
+    gpg,
+    hglib,
+    i18n,
+    paths,
+    terminal,
+    wconfig,
+)
+from ..util.i18n import _
+from . import (
+    customtools,
+    fileencoding,
+    qscilib,
+    qtlib,
+    thgrepo,
+)
 
 if os.name == 'nt':
-    from tortoisehg.util import bugtraq
+    from ..util import bugtraq
     _hasbugtraq = True
 else:
     _hasbugtraq = False
@@ -52,7 +112,7 @@ class SettingsCombo(QComboBox):
             self.previous = []
         else:
             settings = opts['settings']
-            slist = settings.value('settings/'+opts['cpath']).toStringList()
+            slist = qtlib.readStringList(settings, 'settings/' + opts['cpath'])
             self.previous = [unicode(s) for s in slist if s]
         self.setMinimumWidth(ENTRY_WIDTH)
 
@@ -374,7 +434,7 @@ class BugTraqConfigureEntry(QPushButton):
 
         try:
             self.setEnabled(self.tracker.has_options())
-        except Exception, e:
+        except Exception as e:
             qtlib.ErrorMsgBox(_('Issue Tracker'),
                               _('Failed to load issue tracker: \'%s\': %s. '
                                 % (name, e)),
@@ -510,19 +570,19 @@ def issuePluginVisible():
         return False
 
 def findDiffTools():
-    return hglib.difftools(ui.ui())
+    return hglib.difftools(hglib.loadui())
 
 def findMergeTools():
-    return hglib.mergetools(ui.ui())
+    return hglib.mergetools(hglib.loadui())
 
 def findEditors():
-    return editor.findeditors(ui.ui())
+    return editor.findeditors(hglib.loadui())
 
 def findTerminals():
-    return terminal.findterminals(ui.ui())
+    return terminal.findterminals(hglib.loadui())
 
 def findGpg():
-    return gpg.findgpg(ui.ui())
+    return gpg.findgpg(hglib.loadui())
 
 def genCheckBox(opts):
     opts['nohist'] = True
@@ -579,6 +639,11 @@ INFO = (
           'section of your Mercurial configuration files.  If left '
           'unspecified, TortoiseHg will use the first applicable tool '
           'it finds.')),
+    _fi(_('CLI Editor'), 'ui.editor',
+        (genEditableDeferredCombo, findEditors),
+        _('The editor used by Mercurial command line commands to '
+          'collect multiline input from the user.  Most notably, '
+          'commit messages.')),
     _fi(_('Shell'), 'tortoisehg.shell',
         (genEditableDeferredCombo, findTerminals),
         _('Specify the command to launch your preferred terminal shell '
@@ -612,7 +677,8 @@ INFO = (
           'A value of zero implies no limit.  Default: 1024 (1MB)')),
     _fi(_('Fork GUI'), 'tortoisehg.guifork', genBoolRBGroup,
         _('When running from the command line, fork a background '
-          'process to run graphical dialogs.  Default: True'),
+          'process to run graphical dialogs. This setting is ignored when '
+          'TortoiseHg runs as an OS X application bundle.  Default: True'),
         globalonly=True),
     _fi(_('Full Path Title'), 'tortoisehg.fullpath', genBoolRBGroup,
         _('Show a full directory path of the repository in the dialog title '
@@ -629,6 +695,12 @@ INFO = (
         _('If specified, files in the directory, e.g. .hgignore, are copied '
           'to the newly-created repository.'),
         globalonly=True),
+    _fi(_('Default Clone Destination'), 'tortoisehg.defaultclonedest',
+        genPathBrowser,
+        _('If specified, the destination field in the clone widget will be '
+          'pre-filled with this path. Otherwise, it will be pre-filled with '
+          'the current working directory.'),
+        globalonly=True),
     )),
 
 ({'name': 'log', 'label': _('Workbench'), 'icon': 'hg-log'}, (
@@ -720,6 +792,13 @@ INFO = (
           '<b>Note</b>: Calculating family line may be slow in some cases. '
           'This option is expected to be removed if the performance issue is '
           'solved.')),
+    _fi(_('Use optimized graph layouter'), 'tortoisehg.graphopt',
+        genBoolRBGroup,
+        _('Use alternative graph layouter for large repositories. '
+          'Default: False<p>'
+          '<b>Note</b>: This layouter colors edges using branch information '
+          'and does not display graft edges, regardless of whether they are '
+          'requested or not.')),
     )),
 ({'name': 'commit', 'label': _('Commit', 'config item'), 'icon': 'hg-commit'},
  (
@@ -754,7 +833,7 @@ INFO = (
           'This setting is used by the Merge, Tag and Backout dialogs. '
           'Default: False')),
     _fi(_('New Commit Phase'), 'phases.new-commit',
-        (genDefaultCombo, phases.phasenames),
+        (genDefaultCombo, phases.phasenames[:3]),
         _('The phase of new commits. Default: draft')),
     _fi(_('Secret MQ Patches'), 'mq.secret', genBoolRBGroup,
         _('Make MQ patches secret (instead of draft). '
@@ -786,14 +865,14 @@ INFO = (
         genBoolRBGroup,
         _('Determines if TortoiseHg should show a confirmation dialog '
           'before adding new files in a commit. '
-          'If True, a confirmation dialog will be showed. '
+          'If True, a confirmation dialog will be shown. '
           'If False, selected new files will be included in the '
           'commit with no confirmation dialog.  Default: True')),
     _fi(_('Confirm deleting files'), 'tortoisehg.confirmdeletefiles',
         genBoolRBGroup,
         _('Determines if TortoiseHg should show a confirmation dialog '
           'before removing files in a commit. '
-          'If True, a confirmation dialog will be showed. '
+          'If True, a confirmation dialog will be shown. '
           'If False, selected deleted files will be included in the '
           'commit with no confirmation dialog.  Default: True')),
     )),
@@ -831,6 +910,25 @@ INFO = (
     _fi(_('SSH Command'), 'ui.ssh', genEditCombo,
         _('Command to use for SSH connections.<p>'
           'Default: "ssh" or "TortoisePlink.exe -ssh -2" (Windows)')),
+    _fi(_('<b>Subrepository Features:</b>'), None, genSpacer, ''),
+    _fi(_('Allow Hg Subrepos'), 'subrepos.hg:allowed', genBoolRBGroup,
+        _('Whether Mercurial subrepositories are allowed in the working '
+          'directory. '
+          'Default: True')),
+    _fi(_('Allow Git Subrepos'), 'subrepos.git:allowed', genBoolRBGroup,
+        _('Whether Git subrepositories are allowed in the working '
+          'directory. '
+          'Default: False'
+          '<p><a href="%s">See the security note</a> before enabling '
+          'Git subrepos.')
+        % 'https://www.mercurial-scm.org/doc/hgrc.5.html#subrepos'),
+    _fi(_('Allow SVN Subrepos'), 'subrepos.svn:allowed', genBoolRBGroup,
+        _('Whether Subversion subrepositories are allowed in the working '
+          'directory. '
+          'Default: False'
+          '<p><a href="%s">See the security note</a> before enabling '
+          'Subversion subrepos.')
+        % 'https://www.mercurial-scm.org/doc/hgrc.5.html#subrepos'),
     )),
 
 ({'name': 'web', 'label': _('Server'), 'icon': 'hg-serve'}, (
@@ -995,9 +1093,6 @@ INFO = (
     _fi(_('Diff Font'), 'tortoisehg.fontdiff', genFontEdit,
         _('Font used to display text differences. Default: monospace 10'),
         globalonly=True),
-    _fi(_('List Font'), 'tortoisehg.fontlist', genFontEdit,
-        _('Font used to display file lists. Default: sans 9'),
-        globalonly=True),
     _fi(_('ChangeLog Font'), 'tortoisehg.fontlog', genFontEdit,
         _('Font used to display changelog data. Default: monospace 10'),
         globalonly=True),
@@ -1052,22 +1147,29 @@ INFO = (
           'Default: never'),
         master='tortoisehg.issue.bugtraqplugin', visible=issuePluginVisible),
     _fi(_('Changeset Link'), 'tortoisehg.changeset.link', genEditCombo,
-        _('A "template string" that, when set, turns the revision number and '
+        _('A template string that, when set, turns the revision number and '
           'short hashes that are shown on the revision panels into links.<br>'
-          'The "template string" uses a "mercurial template"-like syntax that '
-          'currently accepts two template expressions:'
+          'The template string uses a normal '
+          '<a href="https://www.mercurial-scm.org/repo/hg/help/templates">'
+          'mercurial template syntax</a>, such as:'
           '<ul>'
-          '<li>{node|short} : replaced by the 12 digit revision id (note that '
-          '{node} on its own is currently unsupported).'
+          '<li>{node|short} : replaced by the 12 digit revision id.'
           '<li>{rev} : replaced by the revision number.'
+          '<li>{gitnode} : if the hg-git extension is enabled, and the repo is '
+          'a git clone, this is replaced by the git commit hash.'
           '</ul>'
           'For example, in order to link to bitbucket commit pages you can '
           'set this to:<br>'
-          'https://bitbucket.org/tortoisehg/thg/commits/{node|short}'
+          'https://bitbucket.org/tortoisehg/thg/commits/{node|short}<br>'
+          'You can also to link to a GitHub/GitLab repo (provided hg-git is '
+          'installed): <br>'
+          'https://github.com/torvalds/linux/commit/{gitnode}<br>'
+          'https://gitlab.com/tortoisegit/tortoisegit/commit/{gitnode}<br>'
         )),
     )),
 
-({'name': 'reviewboard', 'label': _('Review Board'), 'icon': 'reviewboard'}, (
+({'name': 'reviewboard', 'label': _('Review Board'), 'icon': 'reviewboard',
+  'extension': 'reviewboard'}, (
     _fi(_('Server'), 'reviewboard.server', genEditCombo,
         _('Path to review board '
           'example "http://demo.reviewboard.org"')),
@@ -1097,6 +1199,13 @@ INFO = (
           'stored')),
     )),
 
+({'name': 'simplelock', 'label': _('Simplelock'), 'icon': 'thg-password',
+  'extension': 'simplelock'}, (
+    _fi(_('Lock Clone'), 'simplelock.repo', genEditCombo,
+        _('Location of local clone of organizational lock repository.<p>'
+          'This repository must contain a "locked" text file')),
+    )),
+
 ({'name': 'largefiles', 'label': _('Largefiles'), 'icon': 'kiln',
   'extension': 'largefiles'}, (
     _fi(_('Patterns'), 'largefiles.patterns', genEditCombo,
@@ -1177,18 +1286,17 @@ class SettingsDialog(QDialog):
             qtlib.ErrorMsgBox(_('Iniparse package not found'),
                          _("Can't change settings without iniparse package - "
                            'view is readonly.'), parent=self)
-            print 'Please install http://code.google.com/p/iniparse/'
+            print('Please install https://code.google.com/archive/p/iniparse/')
 
         if not focus:
-            focus = QSettings().value('settings/lastpage', 'log').toString()
-        focus = unicode(focus)
+            focus = qtlib.readString(QSettings(), 'settings/lastpage', 'log')
 
         layout = QVBoxLayout()
         self.setLayout(layout)
 
         s = QSettings()
         self.settings = s
-        self.restoreGeometry(s.value('settings/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'settings/geom'))
 
         def username():
             name = util.username()
@@ -1202,7 +1310,9 @@ class SettingsDialog(QDialog):
         self._activeformidx = configrepo and CONF_REPO or CONF_GLOBAL
         self.conftabs = QTabWidget()
         layout.addWidget(self.conftabs)
-        utab = SettingsForm(rcpath=scmutil.userrcpath(), focus=focus)
+        if qtlib.IS_RETINA:
+            self.conftabs.setIconSize(qtlib.barRetinaIconSize())
+        utab = SettingsForm(rcpath=hglib.userrcpath(), focus=focus)
         self.conftabs.addTab(utab, qtlib.geticon('thg-userconfig'),
                              _("%s's global settings") % username())
         utab.restartRequested.connect(self._pushRestartRequest)
@@ -1211,7 +1321,7 @@ class SettingsDialog(QDialog):
             if root is None:
                 root = paths.find_root()
             if root:
-                repo = thgrepo.repository(ui.ui(), root)
+                repo = thgrepo.repository(hglib.loadui(), root)
             else:
                 repo = None
         except error.RepoError:
@@ -1253,7 +1363,7 @@ class SettingsDialog(QDialog):
 
     def isDirty(self):
         return any(self.conftabs.widget(i).isDirty()
-                   for i in xrange(self.conftabs.count()))
+                   for i in pycompat.xrange(self.conftabs.count()))
 
     @pyqtSlot(str)
     def _pushRestartRequest(self, key):
@@ -1261,7 +1371,7 @@ class SettingsDialog(QDialog):
 
     def applyChanges(self):
         results = [self.conftabs.widget(i).applyChanges()
-                   for i in xrange(self.conftabs.count())]
+                   for i in pycompat.xrange(self.conftabs.count())]
         if self._restartreqs:
             qtlib.InfoMsgBox(_('Settings'),
                              _('Restart all TortoiseHg applications '
@@ -1359,6 +1469,8 @@ class SettingsForm(QWidget):
         layout.addLayout(bothbox, 8)
         pageList = QListWidget()
         pageList.setResizeMode(QListView.Fixed)
+        if qtlib.IS_RETINA:
+            pageList.setIconSize(qtlib.listviewRetinaIconSize())
         stack = QStackedWidget()
         bothbox.addWidget(pageList, 0)
         bothbox.addWidget(stack, 1)
@@ -1383,9 +1495,7 @@ class SettingsForm(QWidget):
             if isinstance(meta['icon'], str):
                 icon = qtlib.geticon(meta['icon'])
             else:
-                style = QApplication.style()
-                icon = QIcon()
-                icon.addPixmap(style.standardPixmap(meta['icon']))
+                icon = self.style().standardIcon(meta['icon'])
             item = QListWidgetItem(icon, meta['label'])
             pageList.addItem(item)
 
@@ -1538,9 +1648,9 @@ class SettingsForm(QWidget):
                 func = e.values
                 w = func(opts)
             if e.globalonly:
-                w.setEnabled(self.rcpath == scmutil.userrcpath())
+                w.setEnabled(self.rcpath == hglib.userrcpath())
             elif e.noglobal:
-                w.setEnabled(self.rcpath != scmutil.userrcpath())
+                w.setEnabled(self.rcpath != hglib.userrcpath())
             lbl = QLabel(e.label)
             lbl.setToolTip(e.tooltip)
             widgets.append(w)
@@ -1628,9 +1738,8 @@ class SettingsForm(QWidget):
             for fn in rcpath:
                 # Try to create a file from rcpath
                 try:
-                    f = open(fn, 'w')
-                    f.write('# Generated by TortoiseHg settings dialog\n')
-                    f.close()
+                    with open(fn, 'w') as f:
+                        f.write('# Generated by TortoiseHg settings dialog\n')
                     break
                 except (IOError, OSError):
                     pass
@@ -1682,7 +1791,7 @@ class SettingsForm(QWidget):
         try:
             wconfig.writefile(self.ini, self.fn)
             return True
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             qtlib.WarningMsgBox(_('Unable to write configuration file'),
                                 hglib.tounicode(str(e)), parent=self)
             return False
@@ -1747,7 +1856,7 @@ class SettingsForm(QWidget):
                 chk.setEnabled(False)
                 cpath = chk.opts['cpath']
                 sect, key = cpath.split('.', 1)
-                if ui.ui().config(sect, key, None) is not None:
+                if hglib.loadui().config(sect, key, None) is not None:
                     chk.setValue(True)
                     chk.curvalue = True
             else:
