@@ -24,6 +24,7 @@ from .qtgui import (
 
 from mercurial import (
     error,
+    pycompat,
     scmutil,
 )
 
@@ -68,10 +69,11 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             except error.RepoLookupError:
                 pass
 
-        combo.addItems(map(hglib.tounicode, hglib.namedbranches(repo)))
-        tags = list(self.repo.tags()) + repo._bookmarks.keys()
+        combo.addItems(pycompat.maplist(hglib.tounicode,
+                                        hglib.namedbranches(repo)))
+        tags = list(self.repo.tags()) + list(repo._bookmarks.keys())
         tags.sort(reverse=True)
-        combo.addItems(map(hglib.tounicode, tags))
+        combo.addItems(pycompat.maplist(hglib.tounicode, tags))
 
         if rev is None:
             selecturev = hglib.tounicode(self.repo.dirstate.branch())
@@ -384,7 +386,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                 else:
                     return cmdcore.nullCmdSession()
 
-        cmdline = map(hglib.tounicode, cmdline)
+        cmdline = pycompat.maplist(hglib.tounicode, cmdline)
         return self._repoagent.runCommand(cmdline, self)
 
     @pyqtSlot(bool)
