@@ -5,22 +5,46 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
+from __future__ import absolute_import
+
 import os
-import subprocess
-import stat
-import shutil
-import threading
-import tempfile
 import re
+import stat
+import subprocess
+import threading
 
-from mercurial import hg, util, error, match, scmutil, copies
+from .qtcore import (
+    QTimer,
+    pyqtSlot,
+)
+from .qtgui import (
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QHBoxLayout,
+    QKeySequence,
+    QLabel,
+    QListWidget,
+    QMessageBox,
+    QShortcut,
+    QVBoxLayout,
+)
 
-from tortoisehg.util.i18n import _
-from tortoisehg.util import hglib
-from tortoisehg.hgqt import qtlib
+from mercurial import (
+    copies,
+    error,
+    match,
+    pycompat,
+    scmutil,
+    util,
+)
+from mercurial.utils import (
+    procutil,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from ..util import hglib
+from ..util.i18n import _
+from . import qtlib
 
 # Match parent2 first, so 'parent1?' will match both parent1 and parent
 _regex = '\$(parent2|parent1?|child|plabel1|plabel2|clabel|repo|phash1|phash2|chash)'
@@ -104,9 +128,8 @@ def snapshot(repo, files, ctx):
                 os.makedirs(destdir)
             fctx = ctx[wfn]
             data = repo.wwritedata(wfn, fctx.data())
-            f = open(dest, 'wb')
-            f.write(data)
-            f.close()
+            with open(dest, 'wb') as f:
+                f.write(data)
             if 'x' in fctx.flags():
                 util.setflags(dest, False, True)
             if ctx.rev() is None:
@@ -122,7 +145,7 @@ def snapshot(repo, files, ctx):
 def launchtool(cmd, opts, replace, block):
     def quote(match):
         key = match.group()[1:]
-        return util.shellquote(replace[key])
+        return procutil.shellquote(replace[key])
     if isinstance(cmd, unicode):
         cmd = hglib.fromunicode(cmd)
     lopts = []
@@ -133,8 +156,8 @@ def launchtool(cmd, opts, replace, block):
             lopts.append(opt)
     args = ' '.join(lopts)
     args = re.sub(_regex, quote, args)
-    cmdline = util.shellquote(cmd) + ' ' + args
-    cmdline = util.quotecommand(cmdline)
+    cmdline = procutil.shellquote(cmd) + ' ' + args
+    cmdline = procutil.quotecommand(cmdline)
     try:
         proc = subprocess.Popen(cmdline, shell=True,
                                 creationflags=qtlib.openflags,
@@ -143,7 +166,7 @@ def launchtool(cmd, opts, replace, block):
                                 stdin=subprocess.PIPE)
         if block:
             proc.communicate()
-    except (OSError, EnvironmentError), e:
+    except (OSError, EnvironmentError) as e:
         QMessageBox.warning(None,
                 _('Tool launch failure'),
                 _('%s : %s') % (cmd, str(e)))
@@ -175,7 +198,7 @@ def besttool(ui, tools, force=None):
     for t in tools.keys():
         try:
             p = ui.configint('merge-tools', t + '.priority', 0)
-        except error.ConfigError, inst:
+        except error.ConfigError as inst:
             ui.warn('visdiff: %s\n' % inst)
             p = 0
         pris.append((-p, t))
@@ -190,7 +213,11 @@ def visualdiff(ui, repo, pats, opts):
     try:
         ctx1b = None
         if change:
-            ctx2 = repo[change]
+            # TODO: figure out what's the expect type
+            if isinstance(change,  str):
+                ctx2 = scmutil.revsymbol(repo, change)
+            else:
+                ctx2 = repo[change]
             p = ctx2.parents()
             if len(p) > 1:
                 ctx1a, ctx1b = p
@@ -211,9 +238,13 @@ def visualdiff(ui, repo, pats, opts):
     pats = scmutil.expandpats(pats)
     m = match.match(repo.root, '', pats, None, None, 'relpath')
     n2 = ctx2.node()
-    mod_a, add_a, rem_a = map(set, repo.status(ctx1a.node(), n2, m)[:3])
+    mod_a, add_a, rem_a = pycompat.maplist(set,
+                                           repo.status(ctx1a.node(),
+                                                       n2, m)[:3])
     if ctx1b:
-        mod_b, add_b, rem_b = map(set, repo.status(ctx1b.node(), n2, m)[:3])
+        mod_b, add_b, rem_b = pycompat.maplist(set,
+                                               repo.status(ctx1b.node(),
+                                                           n2, m)[:3])
         cpy = copies.mergecopies(repo, ctx1a, ctx1b, ctx1a.ancestor(ctx1b))[0]
     else:
         cpy = copies.pathcopies(ctx1a, ctx2)
@@ -248,7 +279,7 @@ def visualdiff(ui, repo, pats, opts):
         else:
             toollist.add(preferred)
 
-    cto = cpy.keys()
+    cto = list(cpy.keys())
     for path in MAR:
         if path in cto:
             hascopies = True
@@ -428,7 +459,7 @@ class FileSelectionDialog(QDialog):
             hbox.addWidget(lbl)
             hbox.addWidget(combo, 1)
             layout.addLayout(hbox)
-            for i, name in enumerate(tools.iterkeys()):
+            for i, name in enumerate(tools.keys()):
                 combo.addItem(name)
                 if name == preferred:
                     defrow = i
@@ -513,7 +544,7 @@ class FileSelectionDialog(QDialog):
                 break
         else:
             selected = self.preferred
-        for i, name in enumerate(self.tools.iterkeys()):
+        for i, name in enumerate(self.tools.keys()):
             if name == selected:
                 self.toolCombo.setCurrentIndex(i)
 
@@ -528,15 +559,16 @@ class FileSelectionDialog(QDialog):
         self.launch(item.text()[2:])
 
     def updateDiffButtons(self, tool):
+        # hg>=4.4: configbool() may return None as the default is set to None
         if hasattr(self, 'p1button'):
             d2 = self.repo.ui.configbool('merge-tools', tool + '.dirdiff')
             d3 = self.repo.ui.configbool('merge-tools', tool + '.dir3diff')
-            self.p1button.setEnabled(d2)
-            self.p2button.setEnabled(d2)
-            self.p3button.setEnabled(d3)
+            self.p1button.setEnabled(bool(d2))
+            self.p2button.setEnabled(bool(d2))
+            self.p3button.setEnabled(bool(d3))
         elif hasattr(self, 'dbutton'):
             d2 = self.repo.ui.configbool('merge-tools', tool + '.dirdiff')
-            self.dbutton.setEnabled(d2)
+            self.dbutton.setEnabled(bool(d2))
 
     def launch(self, fname):
         fname = hglib.fromunicode(fname)
