@@ -8,19 +8,54 @@
 Main Qt4 application for TortoiseHg
 """
 
+from __future__ import absolute_import
+
 import os
 import subprocess
 import sys
-from tortoisehg.util import paths, hglib
-from tortoisehg.util.i18n import _
 
-from tortoisehg.hgqt import cmdcore, cmdui, qtlib, mq, repotab, serve
-from tortoisehg.hgqt.reporegistry import RepoRegistryView
-from tortoisehg.hgqt.docklog import LogDockWidget
-from tortoisehg.hgqt.settings import SettingsDialog
+from .qtcore import (
+    QSettings,
+    QT_VERSION,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAction,
+    QActionGroup,
+    QApplication,
+    QComboBox,
+    QFileDialog,
+    QKeySequence,
+    QMainWindow,
+    QMenu,
+    QMenuBar,
+    QShortcut,
+    QSizePolicy,
+    QToolBar,
+    qApp,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from mercurial import (
+    pycompat,
+)
+
+from ..util import (
+    hglib,
+    paths,
+)
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    mq,
+    qtlib,
+    repotab,
+    serve,
+)
+from .docklog import LogDockWidget
+from .reporegistry import RepoRegistryView
+from .settings import SettingsDialog
 
 class Workbench(QMainWindow):
     """hg repository viewer/browser application"""
@@ -78,9 +113,16 @@ class Workbench(QMainWindow):
                                     self.cloneRepository)
             self.dockMenu.addAction(_('&Open Repository...'),
                                     self.openRepository)
-            qt_mac_set_dock_menu(self.dockMenu)
-            # On Mac OS X, we do not want icons on menus
-            qt_mac_set_menubar_icons(False)
+            if QT_VERSION < 0x50000:
+                from .qtgui import (
+                    qt_mac_set_dock_menu,
+                    qt_mac_set_menubar_icons,
+                )
+                qt_mac_set_dock_menu(self.dockMenu)
+                # On Mac OS X, we do not want icons on menus
+                qt_mac_set_menubar_icons(False)
+            else:
+                self.dockMenu.setAsDockMenu()
 
         self._dialogs = qtlib.DialogKeeper(
             lambda self, dlgmeth: dlgmeth(self), parent=self)
@@ -102,6 +144,7 @@ class Workbench(QMainWindow):
         tw.currentTitleChanged.connect(self._updateWindowTitle)
         tw.historyChanged.connect(self._updateHistoryActions)
         tw.makeLogVisible.connect(self._setConsoleVisible)
+        tw.taskTabVisibilityChanged.connect(self._updateTaskTabVisibilityAction)
         tw.toolbarVisibilityChanged.connect(self._updateToolBarActions)
 
         self.setCentralWidget(tw)
@@ -154,7 +197,9 @@ class Workbench(QMainWindow):
         newaction(_("&Close Repository"), self.closeCurrentRepoTab,
                   shortcut='Close', enabled='repoopen', menu='file')
         newseparator(menu='file')
-        newaction(_('&Settings'), self.editSettings, icon='settings_user',
+        self.menuFile.addActions(self.repoTabsWidget.tabSwitchActions())
+        newseparator(menu='file')
+        newaction(_('&Settings'), self.editSettings, icon='thg-userconfig',
                   shortcut='Preferences', menu='file')
         newseparator(menu='file')
         newaction(_("E&xit"), self.close, shortcut='Quit', menu='file')
@@ -201,6 +246,11 @@ class Workbench(QMainWindow):
                   menu='view')
         newseparator(menu='view')
 
+        a = newaction(_('Show Tas&k Tab'), shortcut='Alt+0', checkable=True,
+                      enabled='repoopen', menu='view')
+        a.triggered.connect(self._setRepoTaskTabVisible)
+        self.actionTaskTabVisible = a
+
         self.actionGroupTaskView = QActionGroup(self)
         self.actionGroupTaskView.triggered.connect(self._onSwitchRepoTaskTab)
         def addtaskview(icon, label, name):
@@ -214,14 +264,13 @@ class Workbench(QMainWindow):
         # note that 'grep' and 'search' are equivalent
         taskdefs = {
             'commit': ('hg-commit', _('&Commit')),
-            'pbranch': ('branch', _('&Patch Branch')),
+            'pbranch': ('hg-branch', _('&Patch Branch')),
             'log': ('hg-log', _("Revision &Details")),
             'grep': ('hg-grep', _('&Search')),
             'sync': ('thg-sync', _('S&ynchronize')),
             # 'console' is toggled by "Show Console" action
         }
-        tasklist = self.ui.configlist(
-            'tortoisehg', 'workbench.task-toolbar', [])
+        tasklist = self.ui.configlist('tortoisehg', 'workbench.task-toolbar')
         if tasklist == []:
             tasklist = ['log', 'commit', 'grep', 'pbranch', '|', 'sync']
 
@@ -294,17 +343,28 @@ class Workbench(QMainWindow):
 
         newseparator(toolbar='edit')
         menuSync = self.menuRepository.addMenu(_('S&ynchronize'))
+        a = newaction(_("&Lock File..."), self._repofwd('lockTool'),
+                    icon='thg-password', enabled='repoopen',
+                    menu='repository', toolbar='edit',
+                    tooltip=_('Lock or unlock files'))
+        self.lockToolAction = a
         newseparator(menu='repository')
         newaction(_("&Update..."), self._repofwd('updateToRevision'),
                   icon='hg-update', enabled='repoopen',
                   menu='repository', toolbar='edit',
                   tooltip=_('Update working directory or switch revisions'))
-        newaction(_("&Shelve..."), self._repofwd('shelve'), icon='shelve',
-                  enabled='repoopen', menu='repository')
-        newaction(_("&Import..."), self._repofwd('thgimport'), icon='hg-import',
+        newaction(_("&Shelve..."), self._repofwd('shelve'), icon='hg-shelve',
                   enabled='repoopen', menu='repository')
+        newaction(_("&Import Patches..."), self._repofwd('thgimport'),
+                  icon='hg-import', enabled='repoopen', menu='repository')
+        newaction(_("U&nbundle..."), self._repofwd('unbundle'),
+                  icon='hg-unbundle', enabled='repoopen', menu='repository')
         newseparator(menu='repository')
-        newaction(_("&Resolve..."), self._repofwd('resolve'), icon='hg-merge',
+        newaction(_('&Merge...'), self._repofwd('mergeWithOtherHead'),
+                  icon='hg-merge', enabled='repoopen',
+                  menu='repository', toolbar='edit',
+                  tooltip=_('Merge with the other head of the current branch'))
+        newaction(_("&Resolve..."), self._repofwd('resolve'),
                   enabled='repoopen', menu='repository')
         newseparator(menu='repository')
         newaction(_("R&ollback/Undo..."), self._repofwd('rollback'),
@@ -328,7 +388,8 @@ class Workbench(QMainWindow):
         newaction(_("&Terminal"), self.terminal, shortcut='Shift+Ctrl+T',
                   icon='utilities-terminal', enabled='repoopen',
                   menu='repository')
-        newaction(_("&Web Server"), self.serve, menu='repository')
+        newaction(_("&Web Server"), self.serve, menu='repository',
+                  icon='hg-serve')
 
         newaction(_("&Help"), self.onHelp, menu='help', icon='help-browser')
         newaction(_("E&xplorer Help"), self.onHelpExplorer, menu='help')
@@ -339,8 +400,7 @@ class Workbench(QMainWindow):
                   visible=visiblereadme, shortcut='Ctrl+F1')
         newseparator(menu='help')
         newaction(_("About &Qt"), QApplication.aboutQt, menu='help')
-        newaction(_("&About TortoiseHg"), self.onAbout, menu='help',
-                  icon='thg-logo')
+        newaction(_("&About TortoiseHg"), self.onAbout, menu='help', icon='thg')
 
         newaction(_('&Incoming'), data='incoming', icon='hg-incoming',
                   enabled='repoopen', toolbar='sync', shortcut='Ctrl+Shift+,')
@@ -351,6 +411,14 @@ class Workbench(QMainWindow):
         newaction(_('P&ush'), data='push', icon='hg-push',
                   enabled='repoopen', toolbar='sync')
         menuSync.addActions(self.synctbar.actions())
+        menuSync.addSeparator()
+
+        action = QAction(_('&Sync Bookmarks...'), self)
+        action.setIcon(qtlib.geticon('thg-sync-bookmarks'))
+        self._actionavails['repoopen'].append(action)
+        action.triggered.connect(self._runSyncBookmarks)
+        menuSync.addAction(action)
+
         self._lastRepoSyncPath = {}
         self.urlCombo = QComboBox(self)
         self.urlCombo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
@@ -364,7 +432,7 @@ class Workbench(QMainWindow):
         """repository has been switched, fill urlCombo with URLs"""
         pathdict = dict((hglib.tounicode(alias), hglib.tounicode(path))
                          for alias, path in repo.ui.configitems('paths'))
-        aliases = pathdict.keys()
+        aliases = list(pathdict.keys())
 
         combo_setting = repo.ui.config('tortoisehg', 'workbench.target-combo',
                                        'auto')
@@ -424,7 +492,7 @@ class Workbench(QMainWindow):
         self.urlCombo.blockSignals(False)
         self._updateSyncUrlToolTip(self.urlCombo.currentIndex())
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _setupUrlComboIfCurrent(self, root):
         w = self._currentRepoWidget()
         if w.repoRootPath() == root:
@@ -436,7 +504,7 @@ class Workbench(QMainWindow):
         if urlindex < 0:
             return
         opindex = {'incoming': 0, 'pull': 0, 'outgoing': 1, 'push': 1}[op]
-        return self.urlCombo.itemData(urlindex).toPyObject()[opindex]
+        return self.urlCombo.itemData(urlindex)[opindex]
 
     @pyqtSlot(int)
     def _updateSyncUrl(self, index):
@@ -457,7 +525,7 @@ class Workbench(QMainWindow):
             tooltip = _('There are no configured sync paths.\n'
                         'Open the Synchronize tab to configure them.')
         else:
-            tooltip = self.urlCombo.itemData(index, Qt.ToolTipRole).toString()
+            tooltip = self.urlCombo.itemData(index, Qt.ToolTipRole)
         self.urlCombo.setToolTip(tooltip)
 
     def _updateSyncActionToolTip(self, index):
@@ -469,7 +537,7 @@ class Workbench(QMainWindow):
                 'push':     _('Push outgoing changes'),
                 }
         else:
-            pullurl, pushurl = self.urlCombo.itemData(index).toPyObject()
+            pullurl, pushurl = self.urlCombo.itemData(index)
             tooltips = {
                 'incoming': _('Check for incoming changes from\n%s') % pullurl,
                 'pull':     _('Pull incoming changes from\n%s') % pullurl,
@@ -478,7 +546,7 @@ class Workbench(QMainWindow):
                 }
 
         for a in self.synctbar.actions():
-            op = str(a.data().toString())
+            op = str(a.data())
             if op in tooltips:
                 a.setToolTip(tooltips[op])
 
@@ -586,7 +654,14 @@ class Workbench(QMainWindow):
     def _onSwitchRepoTaskTab(self, action):
         rw = self._currentRepoWidget()
         if rw:
-            rw.switchToNamedTaskTab(str(action.data().toString()))
+            rw.switchToNamedTaskTab(str(action.data()))
+
+    @pyqtSlot(bool)
+    def _setRepoTaskTabVisible(self, visible):
+        rw = self._currentRepoWidget()
+        if not rw:
+            return
+        rw.setTaskTabVisible(visible)
 
     @pyqtSlot(bool)
     def _setConsoleVisible(self, visible):
@@ -632,7 +707,7 @@ class Workbench(QMainWindow):
         self._setConsoleTaskTabVisible(not docked and visible)
         self._updateShowConsoleAction()
 
-    @pyqtSlot(QString, bool)
+    @pyqtSlot(str, bool)
     def openRepo(self, root, reuse, bundle=None):
         """Open tab of the specified repo [unicode]"""
         root = unicode(root)
@@ -643,12 +718,12 @@ class Workbench(QMainWindow):
         if not self.repoTabsWidget.openRepo(root, bundle):
             return
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def showRepo(self, root):
         """Activate the repo tab or open it if not available [unicode]"""
         self.openRepo(root, True)
 
-    @pyqtSlot(unicode, QString)
+    @pyqtSlot(str, str)
     def setRevsetFilter(self, path, filter):
         if self.repoTabsWidget.selectRepo(path):
             w = self.repoTabsWidget.currentWidget()
@@ -687,6 +762,7 @@ class Workbench(QMainWindow):
 
         # Update actions affected by repo open/close/change
         self._updateTaskViewMenu()
+        self._updateTaskTabVisibilityAction()
         self._updateToolBarActions()
 
     @pyqtSlot()
@@ -715,19 +791,26 @@ class Workbench(QMainWindow):
                 a.setChecked(False)
             if self.actionSelectTaskPbranch is not None:
                 self.actionSelectTaskPbranch.setVisible(False)
+            self.lockToolAction.setVisible(False)
         else:
             exts = repoWidget.repo.extensions()
             if self.actionSelectTaskPbranch is not None:
                 self.actionSelectTaskPbranch.setVisible('pbranch' in exts)
             name = repoWidget.currentTaskTabName()
             for action in self.actionGroupTaskView.actions():
-                action.setChecked(str(action.data().toString()) == name)
+                action.setChecked(str(action.data()) == name)
+            self.lockToolAction.setVisible('simplelock' in exts)
         self._updateShowConsoleAction()
 
         for i, a in enumerate(a for a in self.actionGroupTaskView.actions()
                               if a.isVisible()):
             a.setShortcut('Alt+%d' % (i + 1))
 
+    @pyqtSlot()
+    def _updateTaskTabVisibilityAction(self):
+        rw = self._currentRepoWidget()
+        self.actionTaskTabVisible.setChecked(bool(rw) and rw.isTaskTabVisible())
+
     @pyqtSlot()
     def _updateHistoryActions(self):
         'Update back / forward actions'
@@ -741,7 +824,7 @@ class Workbench(QMainWindow):
         self._updateMenu()
         self._updateWindowTitle()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _onCurrentRepoChanged(self, curpath):
         curpath = unicode(curpath)
         self._console.setCurrentRepoRoot(curpath or None)
@@ -799,10 +882,19 @@ class Workbench(QMainWindow):
     def _runSyncAction(self, action):
         w = self._currentRepoWidget()
         if w:
-            op = str(action.data().toString())
+            op = str(action.data())
             w.setSyncUrl(self._syncUrlFor(op) or '')
             getattr(w, op)()
 
+    @pyqtSlot()
+    def _runSyncBookmarks(self):
+        w = self._currentRepoWidget()
+        if w:
+            # the sync bookmark dialog is bidirectional but is only able to
+            # handle one remote location therefore we use the push location
+            w.setSyncUrl(self._syncUrlFor('push') or '')
+            w.syncBookmark()
+
     @pyqtSlot()
     def _abortCommands(self):
         root = self.currentRepoRootPath()
@@ -814,7 +906,7 @@ class Workbench(QMainWindow):
     def _updateAbortAction(self, repoagent):
         self.actionAbort.setEnabled(repoagent.isBusy())
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _onBusyChanged(self, root):
         repoagent = self._repomanager.repoAgent(root)
         self._updateAbortAction(repoagent)
@@ -842,9 +934,11 @@ class Workbench(QMainWindow):
                                      _("Goto revision"),
                                      _("Enter revision identifier"))
         if ok:
-            self.gotorev(rev)
+            w = self._currentRepoWidget()
+            assert w
+            w.gotoRev(rev)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def gotorev(self, rev):
         w = self._currentRepoWidget()
         if w:
@@ -864,7 +958,7 @@ class Workbench(QMainWindow):
             self.openRepo(dlg.destination(), False)
 
     @pyqtSlot()
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def cloneRepository(self, uroot=None):
         """ Run clone dialog """
         # it might be better to reuse existing CloneDialog
@@ -881,8 +975,10 @@ class Workbench(QMainWindow):
         dlg.clonedRepository.connect(self._openClonedRepo)
         return dlg
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def _openClonedRepo(self, root, sourceroot):
+        root = unicode(root)
+        sourceroot = unicode(sourceroot)
         self.reporegistry.addClonedRepo(root, sourceroot)
         self.showRepo(root)
 
@@ -1015,33 +1111,33 @@ class Workbench(QMainWindow):
     def restoreSettings(self):
         s = QSettings()
         wb = "Workbench/"
-        self.restoreGeometry(s.value(wb + 'geometry').toByteArray())
-        self.restoreState(s.value(wb + 'windowState').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, wb + 'geometry'))
+        self.restoreState(qtlib.readByteArray(s, wb + 'windowState'))
         self._actionDockedConsole.setChecked(
-            s.value(wb + 'dockedConsole', True).toBool())
+            qtlib.readBool(s, wb + 'dockedConsole', True))
 
         lastreposyncpaths = {}
         npaths = s.beginReadArray('lastreposyncpaths')
         for n in range(npaths):
             s.setArrayIndex(n)
-            root = unicode(s.value('root').toString())
-            lastreposyncpaths[root] = s.value('path').toString()
+            root = qtlib.readString(s, 'root')
+            lastreposyncpaths[root] = qtlib.readString(s, 'path')
         s.endArray()
         self._lastRepoSyncPath = lastreposyncpaths
 
-        save = s.value(wb + 'saveRepos').toBool()
+        save = qtlib.readBool(s, wb + 'saveRepos')
         self.actionSaveRepos.setChecked(save)
-        savelastsyncpaths = s.value(wb + 'saveLastSyncPaths').toBool()
+        savelastsyncpaths = qtlib.readBool(s, wb + 'saveLastSyncPaths')
         self.actionSaveLastSyncPaths.setChecked(savelastsyncpaths)
 
-        openreposvalue = unicode(s.value(wb + 'openrepos').toString())
+        openreposvalue = qtlib.readString(s, wb + 'openrepos')
         if openreposvalue:
             openrepos = openreposvalue.split(',')
         else:
             openrepos = []
         # Note that if a "root" has been passed to the "thg" command,
         # "lastactiverepo" will have no effect
-        lastactiverepo = unicode(s.value(wb + 'lastactiverepo').toString())
+        lastactiverepo = qtlib.readString(s, wb + 'lastactiverepo')
         self.repoTabsWidget.restoreRepos(openrepos, lastactiverepo)
 
         # Clear the lastactiverepo and the openrepos list once the workbench state
@@ -1060,7 +1156,8 @@ class Workbench(QMainWindow):
         lastactiverepo = ''
         if self.actionSaveRepos.isChecked():
             tw = self.repoTabsWidget
-            repostosave = map(tw.repoRootPath, xrange(tw.count()))
+            repostosave = pycompat.maplist(tw.repoRootPath,
+                                           pycompat.xrange(tw.count()))
             lastactiverepo = tw.currentRepoRootPath()
         if not self.repoTabsWidget.closeAllTabs():
             event.ignore()
